#include "camera.h"
#include <GL/glut.h>
#include <string>
#include <iostream>
#include <cmath>
using namespace std;


//The following are default constants used to set up OPENGL projection matrix
const float camera::MyLEFT=-5;		
const float camera::MyRIGHT=5;
const float camera::MyBOTTOM=-5;	
const float camera::MyTOP=5;
const float camera::MyNEAR= 0.1;	
const float camera::MyFAR=50;
const float camera::My_FOV_DEGREE=80;
const float camera::My_ASPECT_RATIO=1; 

//The following are constants measuring a basic unit of change of view
const float camera::UNIT_PERSPECTIVE_FOV_CHANGE	= 1;	//1 degree
const float camera::UNIT_OTHO_CHANGE			= 0.1;		//0.1 unit length

//THe default distance of the camera from the origin
const float camera::DEFAULT_DISTANCE_FROM_ORIGIN = 6.0;

//A speed factor to adjust how fast the camera can revolve around 
const int camera::SPEED_FACTOR = 1;

camera::camera()
{	
	lookedAtPointCoordinate[X] = 0;
	lookedAtPointCoordinate[Y] = 0;
	lookedAtPointCoordinate[Z] = 0;

	upVectorForGluLookAt[X]= 0;
	upVectorForGluLookAt[Y]= 1;
	upVectorForGluLookAt[Z]= 0;

	cameraCoordinate[X] = 3.0*DEFAULT_DISTANCE_FROM_ORIGIN/(5.0*1.414);
	cameraCoordinate[Y] = 4.0*DEFAULT_DISTANCE_FROM_ORIGIN/(5.0*1.414);
	cameraCoordinate[Z] = 5.0*DEFAULT_DISTANCE_FROM_ORIGIN/(5.0*1.414);

	isCameraLookigAtFixedPoint = false;
	isCameraLookigAtFixedDirection = true;

	//The following is the same as useDefaultOthoProjection(); 
	currentProjectionStyle = OTHO;
	left=MyLEFT;		right=MyRIGHT;
	bottom=MyBOTTOM;	top=MyTOP;
	near= MyNEAR;		far=MyFAR;

	fov_degree = My_FOV_DEGREE;
	aspectRatio = My_ASPECT_RATIO; 

	//About aotomatic revolving
	isCameraAutomaticRevolving = false;
	isPhiGoUp = true;;
	previousCameraCoordinate[X] = cameraCoordinate[X];
	previousCameraCoordinate[Y] = cameraCoordinate[Y];
	previousCameraCoordinate[Z] = cameraCoordinate[Z];
	cameraDistanceToLookedAtPoint=0;
	phi = 0; theta = 0;


	//About flight simulation
	isCameraInFlightSimulation = false;

	flightMaxX = 30;
	flightMinX = -30;
	flightMaxY = 30;
	flightMinY = -30;
	flightMaxZ = 30;
	flightMinZ = -30;

	flightSpeed = 0.02;


}

void camera::setCameraNameAndProjectionStyle(string name, PROJECTION_STYLE style)
{
	cameraName = name;
	
	if (style == OTHO)
	{	currentProjectionStyle = OTHO;
		useDefaultOthoProjection();
	}
	else if (style == PERSPECTIVE)
	{	currentProjectionStyle = PERSPECTIVE;
		useDefaultPerspectiveProjection();
	}

}


void camera::resetCameraPosition()
{
	lookedAtPointCoordinate[X] = 0;
	lookedAtPointCoordinate[Y] = 0;
	lookedAtPointCoordinate[Z] = 0;

	cameraCoordinate[X] = 3.0*DEFAULT_DISTANCE_FROM_ORIGIN/(5.0*1.414);
	cameraCoordinate[Y] = 4.0*DEFAULT_DISTANCE_FROM_ORIGIN/(5.0*1.414);
	cameraCoordinate[Z] = 5.0*DEFAULT_DISTANCE_FROM_ORIGIN/(5.0*1.414);

	isCameraInFlightSimulation = false;
	isCameraAutomaticRevolving = false;
	isCameraLookigAtFixedPoint = true;
	isCameraLookigAtFixedDirection = false;
	
	if (currentProjectionStyle == OTHO)
		useDefaultOthoProjection();
	else if (currentProjectionStyle == PERSPECTIVE)
		useDefaultPerspectiveProjection();
	else
	{	currentProjectionStyle = OTHO;
		useDefaultOthoProjection();
	}

}


void camera::displayInfo()
{
}

void camera::useDefaultOthoProjection()
{
	currentProjectionStyle = OTHO;
	left=MyLEFT;		right=MyRIGHT;
	bottom=MyBOTTOM;	top=MyTOP;
	near= MyNEAR;		far=MyFAR;
}


void camera::useDefaultPerspectiveProjection()
{
	currentProjectionStyle = PERSPECTIVE;
	near= MyNEAR;		far=MyFAR;
	fov_degree = My_FOV_DEGREE;
	aspectRatio = My_ASPECT_RATIO; 
}


void camera::do_OpenGL_LookAt_BasedOnThisCamera()
	{
		glMatrixMode(GL_MODELVIEW);
		gluLookAt
		(	cameraCoordinate[X],cameraCoordinate[Y],cameraCoordinate[Z],
			lookedAtPointCoordinate[X],lookedAtPointCoordinate[Y],lookedAtPointCoordinate[Z],
					upVectorForGluLookAt[X], upVectorForGluLookAt[Y], upVectorForGluLookAt[Z]
		);
 	}



void camera::saveToFile_OpenGL_LookAt_BasedOnThisCamera(ofstream & outfile)
	{outfile 
		<<	endl
		<<	" gluLookAt" << endl
		<<	" ( " << cameraCoordinate[X] << ",\t"
		          << cameraCoordinate[Y] << ",\t"
				  << cameraCoordinate[Z] << ",\t"
		<< endl
		<<	"	" << lookedAtPointCoordinate[X] << ",\t"
		          << lookedAtPointCoordinate[Y] << ",\t"
				  << lookedAtPointCoordinate[Z] << ",\t"
		<< endl
		<<  "	0,\t" << "1,\t" << '0'	<< endl
		<<	"  );"			<<endl
		<< endl;
	}



void camera::do_OpenGL_Projection_BasedOnThisCamera(int w, int h)
{
	glViewport(0, 0, w, h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	if (currentProjectionStyle == OTHO)
	{	
		if (w >= h)
			glOrtho(	left * ( float(w)/ float(h)), right * ( float(w)/ float(h)), 
						bottom, top, near, far);
		else
			glOrtho(	left , right , 
						bottom*( float(h)/ float(w)), top* ( float(h)/ float(w)),
						near, far);
	}

	if (currentProjectionStyle == PERSPECTIVE)
		gluPerspective(fov_degree, ( float(w)/ float(h)), near, far);

	glMatrixMode(GL_MODELVIEW);
}


void camera::saveToFile_OpenGL_Projection_BasedOnThisCamera(ofstream & outfile)
{ outfile 
	<<  endl
	<<	"void reshape(int w, int h)"	<< endl
	<<	"{"		<< endl
	<< endl;

 if (currentProjectionStyle == OTHO)
 {
	outfile
	<<	" float left = "	<< left		<< ';' << endl
	<<	" float right = "	<< right	<< ';' << endl
	<<	" float bottom = "	<< bottom	<< ';' << endl
	<<	" float top = "		<< top		<< ';' << endl
	<<	" float near = "	<< near		<< ';' << endl
	<<	" float far = "		<< far		<< ';' << endl
	<< endl
	<<  " glMatrixMode(GL_PROJECTION);" << endl
	<<  " glLoadIdentity();" << endl
	<< endl
	<<  " if (w >= h)"  << endl
	<<	"     glOrtho( left * ( float(w)/ float(h)), right * ( float(w)/ float(h))," << endl
	<<  "              bottom, top," << endl
	<<	"              near, far"	<< endl
	<<	"            );" << endl
	<<  " else"  << endl
	<<	"     glOrtho(  left , right , "  << endl
	<<	"               bottom*( float(h)/ float(w)), top* ( float(h)/ float(w)),"  << endl
	<<	"               near, far"	<< endl
	<<	"            );" << endl
	<< endl
	<<  " glMatrixMode(GL_MODELVIEW);" << endl
	<< endl
	<< endl;
 }

 if (currentProjectionStyle == PERSPECTIVE)
 {
	outfile
	<<	" float near = "		<< near			<< ';' << endl
	<<	" float far = "			<< far			<< ';' << endl
	<<	" float fov_degree = "	<< fov_degree	<< ';' << endl
	<<	" gluPerspective(fov_degree, ( float(w)/ float(h)), near, far);"  
	<< endl
	<< endl;
 }

 outfile
	<<  "}"  << endl
	<<  "//**************  End of reshape function ****************"  << endl
	<< endl;
}


void camera::zoomInCameraView()
{
	if (currentProjectionStyle == OTHO)
	{
		if ( left >=0 || right <= 0 || bottom >= 0 || top <=0)
			return; //THis should never happen

		if (	fabs(left) <= UNIT_OTHO_CHANGE ||	right <= UNIT_OTHO_CHANGE ||
				fabs(bottom) <= UNIT_OTHO_CHANGE ||	top <= UNIT_OTHO_CHANGE 
			)
			return;	//The view is too small to reduce any further

		left	+= UNIT_OTHO_CHANGE;
		right	-= UNIT_OTHO_CHANGE;
		bottom	+= UNIT_OTHO_CHANGE;
		top		-= UNIT_OTHO_CHANGE;
	}
	
	if (currentProjectionStyle == PERSPECTIVE)
	{	if (fov_degree <= 1)
			return;	//The view is too small to reduce any further
		else
			fov_degree -= UNIT_PERSPECTIVE_FOV_CHANGE;
	}

}

void camera::zoomOutCameraView()
{
	if (currentProjectionStyle == OTHO)
	{
		if ( left >=0 || right <= 0 || bottom >= 0 || top <=0)
			return; //THis should never happen

		left	-= UNIT_OTHO_CHANGE;
		right	+= UNIT_OTHO_CHANGE;
		bottom	-= UNIT_OTHO_CHANGE;
		top		+= UNIT_OTHO_CHANGE;
	}
	
	if (currentProjectionStyle == PERSPECTIVE)
	{	fov_degree += UNIT_PERSPECTIVE_FOV_CHANGE;
		if (fov_degree > 160)
			fov_degree  = 160;
	}

}

//*****************************************************************
//**********************  Automatic Camera Revolving **************
//*****************************************************************

void camera::startCameraAutomaticRevolvingAroundTheLookedAtPoint()
{	isCameraAutomaticRevolving = true;
	setupAutomaticRevolving();
};


void camera::stopCameraAutomaticRevolvingAroundTheLookedAtPoint_ReturnToPreviousLocation()
{	for (int i=0; i<3; ++i)
			cameraCoordinate[i] = previousCameraCoordinate[i];
						
	isCameraAutomaticRevolving = false;
}

void camera::stopCameraAutomaticRevolvingAroundTheLookedAtPoint_StayHere()
{	isCameraAutomaticRevolving = false;
}



void camera::setupAutomaticRevolving()
{	const double PI =3.14;

	for (int i=0; i<3; ++i)
			previousCameraCoordinate[i] = cameraCoordinate[i];

	double xDistance = cameraCoordinate[X]-lookedAtPointCoordinate[X];
	double yDistance = cameraCoordinate[Y]-lookedAtPointCoordinate[Y];
	double zDistance = cameraCoordinate[Z]-lookedAtPointCoordinate[Z];

	cameraDistanceToLookedAtPoint
			= sqrt(	xDistance * xDistance +
					yDistance * yDistance +
					zDistance * zDistance 
					);

	phi = asin(	yDistance / cameraDistanceToLookedAtPoint );

	if (zDistance > 0)
	{	theta = acos(	xDistance / 
						sqrt(	xDistance * xDistance +
								zDistance * zDistance	
							)
		  			);
	}
	else
	{	theta = 2* PI -
				acos(	xDistance / 
						sqrt(	xDistance * xDistance +
								zDistance * zDistance	
							)
		  			);
	}
						
	isPhiGoUp = true;
}

void camera::revolveOneStepAroundTheLookedAtPoint()
{	const double PI =3.14;

	theta += (0.01*SPEED_FACTOR);
	if (theta > 2*PI)
	{	theta = 0;
		//cout << "Moving the camera around to look at the scene" << endl
		//	 << "from (" 
		//	 << cameraCoordinate[X] << " " 
		//	 << cameraCoordinate[Y] << " " 
		//	 << cameraCoordinate[Z] << ") to (" 
		//	 << lookedAtPointCoordinate[X] << " " 
		//	 << lookedAtPointCoordinate[Y] << " " 
		//	 << lookedAtPointCoordinate[Z] 
		//	 << endl;

		if (isPhiGoUp) 
		{
			phi += (0.1 *SPEED_FACTOR);
			if (phi > PI/2 )
			{	phi = PI/2;
				isPhiGoUp = false;
			}
		}
		else
		{
			phi -= (0.1 * SPEED_FACTOR);
			if (phi < -PI/2)
			{	phi = -PI/2;
				isPhiGoUp = true;
			}
		}
	}

	cameraCoordinate[Y] = lookedAtPointCoordinate[Y] + cameraDistanceToLookedAtPoint * sin(phi);
	cameraCoordinate[Z] = lookedAtPointCoordinate[Z] + cameraDistanceToLookedAtPoint * cos(phi)* sin(theta);
	cameraCoordinate[X] = lookedAtPointCoordinate[X] + cameraDistanceToLookedAtPoint * cos(phi)* cos(theta);
	
	glutPostRedisplay();
}

//*****************************************************************
//**********************  Flight Simulation ***********************
//*****************************************************************

void camera::normalizeIntoUnitVector(float * aVector, int size)
{
	float sumOfSquares = 0;
	for (int i=0; i<size; i++)
		sumOfSquares += (aVector[i]*aVector[i]);

	float length =	sqrt(	sumOfSquares	);

	if (length == 0)
	{
		aVector[0] = 1;
		for (int i=1; i<size; i++)
			aVector[i] = 0;
	}
	else
	{
		for (int i=0; i<size; i++)
			aVector[i] /= length;
	}

}

void camera::startFlightSimulation
(float maxX, float minX, float maxY, float minY, float maxZ, float minZ)
{	
	//Set the boundaries of the flight
	flightMaxX = maxX;
	flightMinX = minX;
	flightMaxY = maxY;
	flightMinY = minY;
	flightMaxZ = maxZ;
	flightMinZ = minZ;

	isCameraInFlightSimulation = true;
	isCameraLookigAtFixedDirection = false;
	isCameraLookigAtFixedPoint = false;


	for (int i=0; i<3; ++i)
	{
		previousCameraCoordinate[i] = cameraCoordinate[i];
		previousLookedAtPointCoordinate[i] = lookedAtPointCoordinate[i];
	}

	for (int i=0; i<3; ++i)
		unitSpeedVector[i] = lookedAtPointCoordinate[i] - cameraCoordinate[i];
	normalizeIntoUnitVector(unitSpeedVector, 3);

	headPullUpAngle = asin( unitSpeedVector[Z] );
	bodyRotateAngle = 0.05;

	flightSpeed = 0.01;


};


void camera::stopFlightSimulation_ReturnToPreviousLocation()
{	for (int i=0; i<3; ++i)
	{
			cameraCoordinate[i] = previousCameraCoordinate[i];
			lookedAtPointCoordinate[i] = previousLookedAtPointCoordinate[i];
	}

	upVectorForGluLookAt[X]= 0;
	upVectorForGluLookAt[Y]= 1;
	upVectorForGluLookAt[Z]= 0;
						
	isCameraInFlightSimulation = false;
	isCameraLookigAtFixedDirection = true;
	isCameraLookigAtFixedPoint = false;

}

void camera::stopFlightSimulation_StayHere()
{	upVectorForGluLookAt[X]= 0;
	upVectorForGluLookAt[Y]= 1;
	upVectorForGluLookAt[Z]= 0;

	isCameraInFlightSimulation = false;
	isCameraLookigAtFixedDirection = true;
	isCameraLookigAtFixedPoint = false;

}


void camera::flyOneStepInFlightSimulation()
{
	static const float bufferZoneSize = 1.0;
	//Move the camera according to the current speed
	for (int i=0; i<3; ++i)
		cameraCoordinate[i] += ( flightSpeed * unitSpeedVector[i] );

	if ( cameraCoordinate[X] > flightMaxX +  bufferZoneSize)
		cameraCoordinate[X] = flightMinX ;
	if ( cameraCoordinate[X] < flightMinX -  bufferZoneSize)
		cameraCoordinate[X] = flightMaxX ;

	if ( cameraCoordinate[Z] > flightMaxZ +  bufferZoneSize)
		cameraCoordinate[Z] = flightMinZ ;
	if ( cameraCoordinate[Z] < flightMinZ -  bufferZoneSize)
		cameraCoordinate[Z] = flightMaxZ ;

	if ( cameraCoordinate[Y] > flightMaxY +  bufferZoneSize)
		cameraCoordinate[Y] = flightMaxY +  bufferZoneSize ;
	if ( cameraCoordinate[Y] < flightMinY -  bufferZoneSize)
		cameraCoordinate[Y] = flightMinY -  bufferZoneSize ;

	//Update the speed vector and the up vector
	updateFlightSimulationSpeedVector();
	updateFlightSimulationUpVectorForGluLookAt();

	//Set the new looked at point accordingly
	for (int i=0; i<3; ++i)
		lookedAtPointCoordinate[i] = cameraCoordinate[i] + unitSpeedVector[i];

	glutPostRedisplay();
}

void camera::updateFlightSimulationSpeedVector()
{	
	float bodyTurnSpeedVector[3];
	bodyTurnSpeedVector[X]= unitSpeedVector[Z];
	bodyTurnSpeedVector[Z]= -unitSpeedVector[X];
	bodyTurnSpeedVector[Y]= 0;

	unitSpeedVector[X] += ( 0.03*sin(bodyRotateAngle) * bodyTurnSpeedVector[X] );
	unitSpeedVector[Z] += ( 0.03*sin(bodyRotateAngle) * bodyTurnSpeedVector[Z] );
	unitSpeedVector[Y] = 0;

	normalizeIntoUnitVector(unitSpeedVector,3);

	unitSpeedVector[X] *=  cos( headPullUpAngle );
	unitSpeedVector[Z] *=  cos( headPullUpAngle );

	unitSpeedVector[Y] = sin ( headPullUpAngle );
}

void camera::updateFlightSimulationUpVectorForGluLookAt()
{	
	float bodyTurnSpeedVector[3];
	bodyTurnSpeedVector[X]= unitSpeedVector[Z];
	bodyTurnSpeedVector[Y]= 0;
	bodyTurnSpeedVector[Z]= -unitSpeedVector[X];
	normalizeIntoUnitVector(bodyTurnSpeedVector, 3);

	upVectorForGluLookAt[X] = ( sin(bodyRotateAngle) * bodyTurnSpeedVector[X] );
	upVectorForGluLookAt[Z] = ( sin(bodyRotateAngle) * bodyTurnSpeedVector[Z] );

	upVectorForGluLookAt[Y] = cos(bodyRotateAngle);
}


void camera::setFlySimulationParametersAccordingToMouseMotion(double mouseMotionX, double  mouseMotionY)
{
	const double PI =3.14;
	static const float MAX_ANGLE = 85 * PI / 180.00;

	headPullUpAngle += (mouseMotionY * PI / 180.00);
	bodyRotateAngle += (mouseMotionX * PI / 180.00);

	if (headPullUpAngle > MAX_ANGLE)
		headPullUpAngle = MAX_ANGLE;

	if (headPullUpAngle < -MAX_ANGLE)
		headPullUpAngle = -MAX_ANGLE;

	if (bodyRotateAngle > MAX_ANGLE)
		bodyRotateAngle = MAX_ANGLE;

	if (bodyRotateAngle < -MAX_ANGLE)
		bodyRotateAngle = -MAX_ANGLE;
}



//*************************************************************************************


void camera::getRasterPositionLeftTopCorner(float& xPos, float& yPos, float& zPos)
{
	if (currentProjectionStyle == PERSPECTIVE)
	{	
		yPos = near * tan( (3.14/180) * fov_degree / 2.0 ) - 0.005;
		xPos = - (yPos * aspectRatio) ;
		zPos = -near;
	}
	else if (currentProjectionStyle == OTHO)
	{
		xPos = left + 0.3;
		yPos = top - 0.3;
		zPos = -near;
	}
	else
		cout << "Unknown camera projection style" << endl;

}


void camera::getRasterPositionLeftBottomCorner(float& xPos, float& yPos, float& zPos)
{
	if (currentProjectionStyle == PERSPECTIVE)
	{	
		yPos = -near * tan( (3.14/180) * fov_degree / 2.0 ) + 0.005;
		xPos = (yPos * aspectRatio);
		zPos = -near;
	}
	else if (currentProjectionStyle == OTHO)
	{
		xPos = left + 0.3; 
		yPos = bottom + 0.3;
		zPos = -near;
	}
	else
		cout << "Unknown camera projection style" << endl;

}

void camera::saveToFile_CameraInfo(ofstream & outfile)
{

 outfile
//	<<	cameraName	<< endl
//	<< endl
	<<	(int) currentProjectionStyle << endl
		<< endl
	<< cameraCoordinate[X]	<< endl
	<< cameraCoordinate[Y]	<< endl
	<< cameraCoordinate[Z]	<< endl
	<< endl
	<< lookedAtPointCoordinate[X]	<< endl
	<< lookedAtPointCoordinate[Y]	<< endl
	<< lookedAtPointCoordinate[Z]	<< endl
		<< endl
	<< isCameraLookigAtFixedPoint		<< endl
		<< endl
	<< isCameraLookigAtFixedDirection	<< endl
	<<	left	<< endl
	<<	right	<< endl
	<<	bottom	<< endl
	<<	top 	<< endl
	<<	near	<< endl
	<<	far		<< endl
		<< endl
	<<	fov_degree	<< endl
	<<	aspectRatio	<< endl
		<< endl
	<<	isCameraAutomaticRevolving	<< endl
	<<	isPhiGoUp	<< endl
		<< endl
	<<	previousCameraCoordinate[X]	<< endl
	<<	previousCameraCoordinate[Y]	<< endl
	<<	previousCameraCoordinate[Z]	<< endl
		<< endl
	<< cameraDistanceToLookedAtPoint << endl
		<< endl
	<<	phi		<< endl
	<<	theta	<< endl
	<<	endl;
}

void camera::loadFromFile_CameraInfo(ifstream & infile)
{ 
  /*
  char charArrayForCameraName[40];
  while (!infile.eof() && strlen(charArrayForCameraName) <= 0)
	infile.getline( charArrayForCameraName, 40);
  cameraName = charArrayForCameraName;
  */

  int intProjectionStyle;
  infile
	>>	intProjectionStyle;

  currentProjectionStyle = (PROJECTION_STYLE) intProjectionStyle;

  infile
	>> cameraCoordinate[X]	
	>> cameraCoordinate[Y]	
	>> cameraCoordinate[Z]	

	>> lookedAtPointCoordinate[X]	
	>> lookedAtPointCoordinate[Y]	
	>> lookedAtPointCoordinate[Z]	

	>> isCameraLookigAtFixedPoint	

	>> isCameraLookigAtFixedDirection

	>>	left
	>>	right	
	>>	bottom	
	>>	top
	>>	near	
	>>	far	
		
	>>	fov_degree	
	>>	aspectRatio	
		
	>>	isCameraAutomaticRevolving	
	>>	isPhiGoUp
		
	>>	previousCameraCoordinate[X]	
	>>	previousCameraCoordinate[Y]	
	>>	previousCameraCoordinate[Z]	
		
	>> cameraDistanceToLookedAtPoint 
	
	>>	phi		
	>>	theta;

	
 cout
//	<<	cameraName	<< endl
//	<< endl
	<<	currentProjectionStyle << endl
		<< endl
	<< cameraCoordinate[X]	<< endl
	<< cameraCoordinate[Y]	<< endl
	<< cameraCoordinate[Z]	<< endl
	<< "**************** Finish reading one camera *************************" << endl;
}


//Given a windowWidth x windowHeight window and
//	the world coordinates x,y,z of a point
//Determine the window coordinates winX, winY, winZ of the point
//	based on this camera setting
bool camera::projectToScreenCoordinates
	(	int windowWidth,	int windowHeight,
		double x,			double y,		double z,
		double& winX,		double& winY,	double& winZ
	)
{	GLdouble modelMatrix[16];
	GLdouble projMatrix[16];
	GLint viewport[4];

	if (windowWidth <= 0 || windowHeight <= 0)
		return false;

	//Clea both 
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	do_OpenGL_Projection_BasedOnThisCamera(windowWidth,	windowHeight);
	do_OpenGL_LookAt_BasedOnThisCamera();

	glGetDoublev(GL_MODELVIEW_MATRIX, modelMatrix);
	glGetDoublev(GL_PROJECTION_MATRIX, projMatrix);
	glGetIntegerv(GL_VIEWPORT, viewport);

	bool flag = 
		gluProject	(	x, y, z,
						modelMatrix, projMatrix, viewport, 
						& winX, &winY, & winZ 
					) ;

	return flag;
}



void camera::moveCameraAccordingToMouseMotion(double mouseMotionX, double  mouseMotionY)
{
	//Determine the unit view vector from the camera to the looked at point
	float viewVector[3];
	viewVector[X] = lookedAtPointCoordinate[X] - cameraCoordinate[X];
	viewVector[Y] = lookedAtPointCoordinate[Y] - cameraCoordinate[Y];
	viewVector[Z] = lookedAtPointCoordinate[Z] - cameraCoordinate[Z];

	float lengthViewVector =
					sqrt(	viewVector[X]*viewVector[X] + 
							viewVector[Y]*viewVector[Y] +
							viewVector[Z]*viewVector[Z] 
						);

	if (lengthViewVector < 0.001)
		return;

	viewVector[X] /= lengthViewVector;
	viewVector[Y] /= lengthViewVector;
	viewVector[Z] /= lengthViewVector;


	//**************  Determine the unit vector moving to the right of camera
	float moveRightVector[3];
	moveRightVector[X] = -viewVector[Z];
	moveRightVector[Y] = 0;
	moveRightVector[Z] = viewVector[X];

	float lengthMoveRightVector;
	lengthMoveRightVector	=
					sqrt(	moveRightVector[X]*moveRightVector[X] + 
							moveRightVector[Y]*moveRightVector[Y] +
							moveRightVector[Z]*moveRightVector[Z] 
						);

	if (lengthMoveRightVector < 0.001)
		return;

	moveRightVector[X] /= lengthMoveRightVector;
	moveRightVector[Y] /= lengthMoveRightVector;
	moveRightVector[Z] /= lengthMoveRightVector;


	//***  Update camera location according to the mouse motion ***
	float moveRightUnit = mouseMotionX/10.0;
	float moveTowardUnit = mouseMotionY/6.0;
	if (moveTowardUnit > 0.95 * lengthViewVector)
		moveTowardUnit = 0.95 * lengthViewVector;

	moveCameraAlongAxis	(	X,
							moveRightUnit	*	moveRightVector[X] + 
							moveTowardUnit	*	viewVector[X]
						);

	moveCameraAlongAxis	(	Y,
							moveRightUnit	*	moveRightVector[Y] + 
							moveTowardUnit	*	viewVector[Y]
						);

	moveCameraAlongAxis	(	Z,
							moveRightUnit	*	moveRightVector[Z] + 
							moveTowardUnit	*	viewVector[Z]
						);

}

