#include "geometricObject.h"
#include <GL/glut.h>
#include <iostream>
using namespace std;

geometricObject::geometricObject()
{	prototype = base_GLUT_WIRE_CUBE;

	coordinate[X] = 0.7 * (rand() % 3);
	coordinate[Y] = 0.7 * (rand() % 3);
	coordinate[Z] = 0.7 * (rand() % 3);

	rotate[X] = rotate[Y] = rotate[Z] = 0;

	scale[X] = scale[Y] = scale[Z] = 0.5;

	color[R] = (rand() % 3)/2.0;
	color[G] = (rand() % 3)/2.0;
	color[B] = (rand() % 3)/2.0;
	color[A] = 1.0;
}

void geometricObject::displayInfo()
{	
	cout << "******  Object Info ********" << endl;
	cout << "(coordinate[X], coordinate[Y], coordinate[Z])= ("
		 << coordinate[X] << ',' << coordinate[Y] << ','<< coordinate[Z] << ')'
		 << endl;
	cout << "(rotate[X], rotate[Y], rotate[Z])= ("
		 << rotate[X] << ',' << rotate[Y] << ','<< rotate[Z] << ')'
		 << endl;
	cout << "(scale[X], scale[Y], scale[Z])= ("
		 << scale[X] << ',' << scale[Y] << ','<< scale[Z] << ')'
		 << endl;
	cout << "Color= ("
		 << color[R] << ',' << color[G] << ','<< color[B]<< ')'
		 << endl
		 << endl;
}

void geometricObject::moveAlongAxis(XYZ_AXIS axis, float units)
{ coordinate[axis] += units;
}


void geometricObject::rotateAlongAxis(XYZ_AXIS axis, float degrees)
{  rotate[axis] = int(rotate[axis]+ degrees) % 360;
}

void geometricObject::scaleAlongAxis(XYZ_AXIS axis, float fraction)
{	if (scale[axis] + fraction > 0.0)
		scale[axis] += fraction; 
}

void geometricObject::scaleColorAlongAxis(COLOR_AXIS axis, float fraction)
{	color[axis] += fraction; 
	if (color[axis]  < 0.0) 
		color[axis] = 0.0;
	else if (color[axis] > 1.0)
		color[axis] = 1.0;
}



bool geometricObject::isModeledInWirePrototype()
{
	switch(prototype)
	{
		case base_GLUT_WIRE_SPHERE:
		case base_GLUT_WIRE_CUBE:
		case base_GLUT_WIRE_CONE:
		case base_GLUT_WIRE_TORUS:
		case base_GLUT_WIRE_TEAPOT:
		case base_GLU_SPHERE_LINE:
		case base_GLU_CYLINDER_LINE:
		case base_GLU_DISK_LINE:
		case base_GLU_PARTIALDISK_LINE:
			return true;
			break;

		case base_GLUT_SOLID_SPHERE:
		case base_GLUT_SOLID_CUBE:
		case base_GLUT_SOLID_CONE:
		case base_GLUT_SOLID_TORUS:
		case base_GLUT_SOLID_TEAPOT:
		case base_GLU_SPHERE_FILL:
		case base_GLU_CYLINDER_FILL:
		case base_GLU_DISK_FILL:
		case base_GLU_PARTIALDISK_FILL:
			return false;
			break;

		default:
			return true;
			break;

	}
}


bool geometricObject::isModeledInSolidPrototype()
{
	switch(prototype)
	{
		case base_GLUT_WIRE_SPHERE:
		case base_GLUT_WIRE_CUBE:
		case base_GLUT_WIRE_CONE:
		case base_GLUT_WIRE_TORUS:
		case base_GLUT_WIRE_TEAPOT:
		case base_GLU_SPHERE_LINE:
		case base_GLU_CYLINDER_LINE:
		case base_GLU_DISK_LINE:
		case base_GLU_PARTIALDISK_LINE:
			return false;
			break;

		case base_GLUT_SOLID_SPHERE:
		case base_GLUT_SOLID_CUBE:
		case base_GLUT_SOLID_CONE:
		case base_GLUT_SOLID_TORUS:
		case base_GLUT_SOLID_TEAPOT:
		case base_GLU_SPHERE_FILL:
		case base_GLU_CYLINDER_FILL:
		case base_GLU_DISK_FILL:
		case base_GLU_PARTIALDISK_FILL:
			return true;
			break;

		default:
			return false;
			break;

	}
}


void geometricObject::doOpenGLModelTransformations()
{
		glTranslatef(	coordinate[X],
						coordinate[Y],
						coordinate[Z]
					);

		glRotatef(rotate[Z], 0.0, 0.0, 1.0);
		glRotatef(rotate[Y], 0.0, 1.0, 0.0);
		glRotatef(rotate[X], 1.0, 0.0, 0.0);

		glScalef(	scale[X],
					scale[Y],
					scale[Z]
				);
}


void geometricObject::saveToFile_OpenGLModelTransformations(ofstream & outfile)
{
	outfile 
	<<  endl
	<<	" glTranslatef("	<< coordinate[X] << ", "	
							<< coordinate[Y] << ", "
							<< coordinate[Z] << ");"	  << endl
	<<  endl
	<<	" glRotatef(" << rotate[Z] << ", 0.0, 0.0, 1.0);" << endl 
	<<	" glRotatef(" << rotate[Y] << ", 0.0, 1.0, 0.0);" << endl 
	<<	" glRotatef(" << rotate[X] << ", 1.0, 0.0, 0.0);" << endl 
	<<  endl
	<<	" glScalef("	<< scale[X] << ","	
						<< scale[Y] << ","	
						<< scale[Z]	<< ");" << endl
	<<  endl;
}

void geometricObject::setOpenGLColorAndMaterialProperties()
{	if ( isModeledInWirePrototype() )
		glColor4fv(color);
	else
	{
		glMaterialfv(GL_FRONT, GL_AMBIENT, color);
		glMaterialfv(GL_FRONT, GL_DIFFUSE, color);
	}
}

void geometricObject::saveToFile_SetOpenGLColor(ofstream & outfile)
{	outfile << endl 
			<< " glColor3f(" << color[0] << ',' 
							 << color[1] << ',' 
							 << color[2] << ");" 
			<< endl 
			<< endl;
}




//A global static object used to store information to draw 
//	some standard glut and dlut 3D geometric objects
//GLUquadricObj quadricInfo;

//A global static pointer to the the object above
static GLUquadricObj * ptrToQuadricInfo;
static bool quadricInfoAllocated = false;

void Xinit(void)
{
   GLfloat ambient[] = { 0.0, 0.0, 0.0, 1.0 };
   GLfloat diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
   GLfloat specular[] = { 1.0, 1.0, 1.0, 1.0 };
   GLfloat position[] = { 0.0, 3.0, 2.0, 0.0 };
   GLfloat lmodel_ambient[] = { 0.4, 0.4, 0.4, 1.0 };
   GLfloat local_view[] = { 0.0 };

   
	GLfloat no_mat[] = { 0.0, 0.0, 0.0, 1.0 };
	GLfloat mat_ambient[] = { 0.7, 0.7, 0.7, 1.0 };
	GLfloat mat_ambient_color[] = { 0.8, 0.8, 0.2, 1.0 };
	GLfloat mat_diffuse[] = { 0.1, 0.5, 0.8, 1.0 };
	GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
   	GLfloat no_shininess[] = { 0.0 };
	GLfloat low_shininess[] = { 5.0 };
	GLfloat high_shininess[] = { 100.0 };
	GLfloat mat_emission[] = {0.3, 0.2, 0.2, 0.0};


   glClearColor(0.0, 0.1, 0.1, 0.0);
   glEnable(GL_DEPTH_TEST);
   glShadeModel(GL_SMOOTH);

   glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
   glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
   glLightfv(GL_LIGHT0, GL_POSITION, position);
   glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
   glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, local_view);

   glEnable(GL_LIGHTING);
   glEnable(GL_LIGHT0);

   			glMaterialfv(GL_FRONT, GL_AMBIENT, no_mat);
			glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
			glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
			glMaterialfv(GL_FRONT, GL_SHININESS, high_shininess);
			glMaterialfv(GL_FRONT, GL_EMISSION, no_mat);
  
			glutSolidTeapot(1);

}

void xinit(void)
{
   GLfloat ambient[] = { 0.0, 0.0, 0.0, 1.0 };
   GLfloat diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
   GLfloat specular[] = { 1.0, 1.0, 1.0, 1.0 };
   GLfloat position[] = { 0.0, 3.0, 2.0, 0.0 };
   GLfloat lmodel_ambient[] = { 0.4, 0.4, 0.4, 1.0 };
   GLfloat local_view[] = { 0.0 };

   glClearColor(0.0, 0.1, 0.1, 0.0);
   glEnable(GL_DEPTH_TEST);
   glShadeModel(GL_SMOOTH);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();

   glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
   glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
   glLightfv(GL_LIGHT0, GL_POSITION, position);
   glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
   glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, local_view);

   glEnable(GL_LIGHTING);
   glEnable(GL_LIGHT0);
}

void xdisplay(void)
{    xinit();

   GLfloat no_mat[] = { 0.0, 0.0, 0.0, 1.0 };
   GLfloat mat_ambient[] = { 0.7, 0.7, 0.7, 1.0 };
   GLfloat mat_ambient_color[] = { 0.8, 0.8, 0.2, 1.0 };
   GLfloat mat_diffuse[] = { 0.1, 0.5, 0.8, 1.0 };
   GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
   GLfloat no_shininess[] = { 0.0 };
   GLfloat low_shininess[] = { 5.0 };
   GLfloat high_shininess[] = { 100.0 };
   GLfloat mat_emission[] = {0.3, 0.2, 0.2, 0.0};

   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

/*  draw sphere in first row, first column
 *  diffuse reflection only; no ambient or specular  
 */
   glPushMatrix();
   glTranslatef (-3.75, 3.0, 0.0);
   glMaterialfv(GL_FRONT, GL_AMBIENT, no_mat);
   glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
   glMaterialfv(GL_FRONT, GL_SPECULAR, no_mat);
   glMaterialfv(GL_FRONT, GL_SHININESS, no_shininess);
   glMaterialfv(GL_FRONT, GL_EMISSION, no_mat);
   glutSolidSphere(1, 10, 10);
   glPopMatrix();

/*  draw sphere in first row, second column
 *  diffuse and specular reflection; low shininess; no ambient
 */
   glPushMatrix();
   glTranslatef (-1.25, 3.0, 0.0);
   glMaterialfv(GL_FRONT, GL_AMBIENT, no_mat);
   glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
   glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
   glMaterialfv(GL_FRONT, GL_SHININESS, low_shininess);
   glMaterialfv(GL_FRONT, GL_EMISSION, no_mat);
   glutSolidTeapot(1);
   glPopMatrix();

/*  draw sphere in first row, third column
 *  diffuse and specular reflection; high shininess; no ambient
 */
   glPushMatrix();
   glTranslatef (1.25, 3.0, 0.0);
   glMaterialfv(GL_FRONT, GL_AMBIENT, no_mat);
   glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
   glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
   glMaterialfv(GL_FRONT, GL_SHININESS, high_shininess);
   glMaterialfv(GL_FRONT, GL_EMISSION, no_mat);
   glutSolidTeapot(1);
   glPopMatrix();

/*  draw sphere in first row, fourth column
 *  diffuse reflection; emission; no ambient or specular reflection
 */
   glPushMatrix();
   glTranslatef (3.75, 3.0, 0.0);
   glMaterialfv(GL_FRONT, GL_AMBIENT, no_mat);
   glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
   glMaterialfv(GL_FRONT, GL_SPECULAR, no_mat);
   glMaterialfv(GL_FRONT, GL_SHININESS, no_shininess);
   glMaterialfv(GL_FRONT, GL_EMISSION, mat_emission);
   glutSolidTeapot(1);
   glPopMatrix();

/*  draw sphere in second row, first column
 *  ambient and diffuse reflection; no specular  
 */
   glPushMatrix();
   glTranslatef (-3.75, 0.0, 0.0);
   glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
   glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
   glMaterialfv(GL_FRONT, GL_SPECULAR, no_mat);
   glMaterialfv(GL_FRONT, GL_SHININESS, no_shininess);
   glMaterialfv(GL_FRONT, GL_EMISSION, no_mat);
   glutSolidTeapot(1);
   glPopMatrix();

/*  draw sphere in second row, second column
 *  ambient, diffuse and specular reflection; low shininess
 */
   glPushMatrix();
   glTranslatef (-1.25, 0.0, 0.0);
   glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
   glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
   glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
   glMaterialfv(GL_FRONT, GL_SHININESS, low_shininess);
   glMaterialfv(GL_FRONT, GL_EMISSION, no_mat);
   glutSolidTeapot(1);
   glPopMatrix();

/*  draw sphere in second row, third column
 *  ambient, diffuse and specular reflection; high shininess
 */
   glPushMatrix();
   glTranslatef (1.25, 0.0, 0.0);
   glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
   glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
   glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
   glMaterialfv(GL_FRONT, GL_SHININESS, high_shininess);
   glMaterialfv(GL_FRONT, GL_EMISSION, no_mat);
   glutSolidTeapot(1);
   glPopMatrix();

/*  draw sphere in second row, fourth column
 *  ambient and diffuse reflection; emission; no specular
 */
   glPushMatrix();
   glTranslatef (3.75, 0.0, 0.0);
   glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
   glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
   glMaterialfv(GL_FRONT, GL_SPECULAR, no_mat);
   glMaterialfv(GL_FRONT, GL_SHININESS, no_shininess);
   glMaterialfv(GL_FRONT, GL_EMISSION, mat_emission);
   glutSolidTeapot(1);
   glPopMatrix();

/*  draw sphere in third row, first column
 *  colored ambient and diffuse reflection; no specular  
 */
   glPushMatrix();
   glTranslatef (-3.75, -3.0, 0.0);
   glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient_color);
   glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
   glMaterialfv(GL_FRONT, GL_SPECULAR, no_mat);
   glMaterialfv(GL_FRONT, GL_SHININESS, no_shininess);
   glMaterialfv(GL_FRONT, GL_EMISSION, no_mat);
   glutSolidTeapot(1);
   glPopMatrix();

/*  draw sphere in third row, second column
 *  colored ambient, diffuse and specular reflection; low shininess
 */
   glPushMatrix();
   glTranslatef (-1.25, -3.0, 0.0);
   glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient_color);
   glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
   glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
   glMaterialfv(GL_FRONT, GL_SHININESS, low_shininess);
   glMaterialfv(GL_FRONT, GL_EMISSION, no_mat);
   glutSolidTeapot(1);
   glPopMatrix();

/*  draw sphere in third row, third column
 *  colored ambient, diffuse and specular reflection; high shininess
 */
   glPushMatrix();
   glTranslatef (1.25, -3.0, 0.0);
   glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient_color);
   glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
   glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
   glMaterialfv(GL_FRONT, GL_SHININESS, high_shininess);
   glMaterialfv(GL_FRONT, GL_EMISSION, no_mat);
   glutSolidTeapot(1);
   glPopMatrix();

/*  draw sphere in third row, fourth column
 *  colored ambient and diffuse reflection; emission; no specular
 */
   glPushMatrix();
   glTranslatef (3.75, -3.0, 0.0);
   glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient_color);
   glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
   glMaterialfv(GL_FRONT, GL_SPECULAR, no_mat);
   glMaterialfv(GL_FRONT, GL_SHININESS, no_shininess);
   glMaterialfv(GL_FRONT, GL_EMISSION, mat_emission);
   glutSolidTeapot(1);
   glPopMatrix();

   glFlush();
}

void geometricObject::renderPrototypeInOpenGL()
{	//xdisplay();
	//return;

	if ( !quadricInfoAllocated )
	{	ptrToQuadricInfo = gluNewQuadric();
		quadricInfoAllocated = true;
	}

	if ( isModeledInSolidPrototype() )
		glEnable(GL_LIGHTING);
	else
		glDisable(GL_LIGHTING);


	GLfloat no_mat[] = { 0.0, 0.0, 0.0, 1.0 };
	GLfloat mat_ambient[] = { 0.7, 0.7, 0.7, 1.0 };
	GLfloat mat_ambient_color[] = { 0.8, 0.8, 0.2, 1.0 };
	GLfloat mat_diffuse[] = { 0.1, 0.5, 0.8, 1.0 };
	GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
	GLfloat no_shininess[] = { 0.0 };
	GLfloat low_shininess[] = { 5.0 };
	GLfloat high_shininess[] = { 100.0 };
	GLfloat mat_emission[] = {0.3, 0.2, 0.2, 0.0};

	switch(prototype)
	{
		case base_GLUT_WIRE_SPHERE:
			glutWireSphere(1, NUM_SLICES, NUM_STACKS);
			break;
		case base_GLUT_SOLID_SPHERE:
			glutSolidSphere(1, NUM_SLICES, NUM_STACKS);
			break;
		case base_GLUT_WIRE_CUBE:
			glutWireCube(1);
			break;
		case base_GLUT_SOLID_CUBE:
			glutSolidCube(1);
			break;
		case base_GLUT_WIRE_CONE:
			glutWireCone(1, 1, NUM_SLICES, NUM_STACKS);
			break;
		case base_GLUT_SOLID_CONE:
			glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
			glutSolidCone(1, 1, NUM_SLICES, NUM_STACKS);
			glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
			break;
		case base_GLUT_WIRE_TORUS:
			glutWireTorus(0.2, 1.0, NUM_SIDES, NUM_RINGS);
			break;
		case base_GLUT_SOLID_TORUS:
			glutSolidTorus(0.2, 1.0, NUM_SIDES, NUM_RINGS);
			break;
		case base_GLUT_WIRE_TEAPOT:
			glutWireTeapot(1);
			break;
		case base_GLUT_SOLID_TEAPOT:
			glutSolidTeapot(1);
			break;

		case base_GLU_SPHERE_LINE:
		 	gluQuadricDrawStyle(ptrToQuadricInfo, GLU_LINE);
			gluSphere(ptrToQuadricInfo, 1, NUM_SLICES, NUM_STACKS);
			break;
		case base_GLU_SPHERE_FILL:
		 	gluQuadricDrawStyle(ptrToQuadricInfo, GLU_FILL);
			gluSphere(ptrToQuadricInfo, 1, NUM_SLICES, NUM_STACKS);
			break;
		case base_GLU_CYLINDER_LINE:
		 	gluQuadricDrawStyle(ptrToQuadricInfo, GLU_LINE);
			gluCylinder(ptrToQuadricInfo,
						2,1,1, NUM_SLICES, NUM_STACKS);
			break;
		case base_GLU_CYLINDER_FILL:
		 	gluQuadricDrawStyle(ptrToQuadricInfo, GLU_FILL);
			gluCylinder(ptrToQuadricInfo,
						2,1,1, NUM_SLICES, NUM_STACKS);
			break;
		case base_GLU_DISK_LINE:
		 	gluQuadricDrawStyle(ptrToQuadricInfo, GLU_LINE);
			gluDisk(ptrToQuadricInfo,
						1, 1.2, NUM_SIDES, NUM_RINGS);
			break;
		case base_GLU_DISK_FILL:
		 	gluQuadricDrawStyle(ptrToQuadricInfo, GLU_FILL);
			gluDisk(ptrToQuadricInfo,
						1, 1.2, NUM_SIDES, NUM_RINGS);
			break;
		case base_GLU_PARTIALDISK_LINE:
		 	gluQuadricDrawStyle(ptrToQuadricInfo, GLU_LINE);
			gluPartialDisk(ptrToQuadricInfo,
						1, 1.2, NUM_SIDES, NUM_RINGS, 0, 30);
			break;
		case base_GLU_PARTIALDISK_FILL:
		 	gluQuadricDrawStyle(ptrToQuadricInfo, GLU_FILL);
			gluPartialDisk(ptrToQuadricInfo,
						1, 1.2, NUM_SIDES, NUM_RINGS, 0, 30);
			break;
		default:
			glutWireCube(1);
			break;

	}
}


void geometricObject::rendeOriginalRefereceFrameOfPrototype()
{

	glDisable(GL_LIGHTING);

	float lengthX, lengthY, lengthZ;
	lengthX = 0.5 +  1.5 / scale[X];
	lengthY = 0.5 +  1.5 / scale[Y];
	lengthZ = 0.5 +  1.5 / scale[Z];

	//Draw XYZ axes
	glBegin(GL_LINES);
		glColor3f(1,0,0);
		glVertex3f(0,0,0);
		glVertex3f(lengthX,0,0);

		glColor3f(0,1,0);
		glVertex3f(0,0,0);
		glVertex3f(0,lengthY,0);

		glColor3f(0,0,1);
		glVertex3f(0,0,0);
		glVertex3f(0,0,lengthZ);
	glEnd();

	//Show the X, Y, Z letters
	glColor3f(1, 0, 0);
	glRasterPos3f(lengthX, 0, 0);
	glutBitmapCharacter(GLUT_BITMAP_8_BY_13, 'x');

	glColor3f(0, 1, 0);
	glRasterPos3f(0, lengthY, 0);
	glutBitmapCharacter(GLUT_BITMAP_8_BY_13, 'y');

	glColor3f(0, 0, 1);
	glRasterPos3f(0, 0, lengthZ);
	glutBitmapCharacter(GLUT_BITMAP_8_BY_13, 'z');

}


void geometricObject::renderCursorInOpenGL()
{	float CURSOR_DIFFUSE[] = {1.0, 1.0, 1.0, 0.5};
	if ( !quadricInfoAllocated )
	{	ptrToQuadricInfo = gluNewQuadric();
		quadricInfoAllocated = true;
	}


	glDisable(GL_LIGHTING);
	glColor4f(1.0, 1.0, 1.0, 0.5);

	if ( isModeledInSolidPrototype() )
	{
			glScalef(1.4, 1.4, 1.4);
			glutWireCube(1);
	}
	else
	switch(prototype)
	{
		case base_GLUT_WIRE_SPHERE:
			glutSolidSphere(1, NUM_SLICES, NUM_STACKS);
			break;
		case base_GLUT_SOLID_SPHERE:
			glutWireSphere(1, NUM_SLICES, NUM_STACKS);
			break;

		case base_GLUT_WIRE_CUBE:
			glutSolidCube(1);
			break;
		case base_GLUT_SOLID_CUBE:
			glutWireCube(1);
			break;

		case base_GLUT_WIRE_CONE:
			glutSolidCone(1, 1, NUM_SLICES, NUM_STACKS);
			break;
		case base_GLUT_SOLID_CONE:
			glutWireCone(1, 1, NUM_SLICES, NUM_STACKS);
			break;

		case base_GLUT_WIRE_TORUS:
			glutSolidTorus(1, 0.5, NUM_SIDES, NUM_RINGS);
			break;
		case base_GLUT_SOLID_TORUS:
			glutWireTorus(1, 0.5, NUM_SIDES, NUM_RINGS);
			break;

		case base_GLUT_WIRE_TEAPOT:
			glutSolidTeapot(1);
			break;
		case base_GLUT_SOLID_TEAPOT:
			glutWireTeapot(1);
			break;

		case base_GLU_SPHERE_LINE:
		 	gluQuadricDrawStyle(ptrToQuadricInfo, GLU_FILL);
			gluSphere(ptrToQuadricInfo, 1, NUM_SLICES, NUM_STACKS);
			break;

		case base_GLU_SPHERE_FILL:
		 	gluQuadricDrawStyle(ptrToQuadricInfo, GLU_LINE);
			gluSphere(ptrToQuadricInfo, 1, NUM_SLICES, NUM_STACKS);
			break;

		case base_GLU_CYLINDER_LINE:
		 	gluQuadricDrawStyle(ptrToQuadricInfo, GLU_FILL);
			gluCylinder(ptrToQuadricInfo,
						2,1,1, NUM_SLICES, NUM_STACKS);
			break;
		case base_GLU_CYLINDER_FILL:
		 	gluQuadricDrawStyle(ptrToQuadricInfo, GLU_LINE);
			gluCylinder(ptrToQuadricInfo,
						2,1,1, NUM_SLICES, NUM_STACKS);
			break;

		case base_GLU_DISK_LINE:
		 	gluQuadricDrawStyle(ptrToQuadricInfo, GLU_FILL);
			gluDisk(ptrToQuadricInfo,
						1, 1.2, NUM_SIDES, NUM_RINGS);
			break;
		case base_GLU_DISK_FILL:
		 	gluQuadricDrawStyle(ptrToQuadricInfo, GLU_LINE);
			gluDisk(ptrToQuadricInfo,
						1, 1.2, NUM_SIDES, NUM_RINGS);
			break;

		case base_GLU_PARTIALDISK_LINE:
		 	gluQuadricDrawStyle(ptrToQuadricInfo, GLU_FILL);
			gluPartialDisk(ptrToQuadricInfo,
						1, 1.2, NUM_SIDES, NUM_RINGS, 0, 30);
			break;
		case base_GLU_PARTIALDISK_FILL:
		 	gluQuadricDrawStyle(ptrToQuadricInfo, GLU_LINE);
			gluPartialDisk(ptrToQuadricInfo,
						1, 1.2, NUM_SIDES, NUM_RINGS, 0, 30);
			break;
		default:
			glutWireCube(1);
			break;

	}
}


//The following an alternative version of cursor
//****************************************************************
//void geometricObject::renderCursorInOpenGL()
//{	float CURSOR_DIFFUSE[] = {1.0, 1.0, 1.0, 0.5};
//	if ( !quadricInfoAllocated )
//	{	ptrToQuadricInfo = gluNewQuadric();
//		quadricInfoAllocated = true;
//	}

	//if ( isModeledInSolidPrototype() )
	//	{
	//		glEnable(GL_LIGHTING);
	//		glScalef(1.05, 1.05, 1.05);
	//		glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, CURSOR_DIFFUSE);

	//	}
	//else
	//	{	glDisable(GL_LIGHTING);
	//		glColor4f(1.0, 1.0, 1.0, 0.5);
	//	}
	//
	//switch(prototype)
	//{
	//	case base_GLUT_WIRE_SPHERE:
	//	case base_GLUT_SOLID_SPHERE:
	//		glutSolidSphere(1, NUM_SLICES, NUM_STACKS);
	//		break;

	//	case base_GLUT_WIRE_CUBE:
	//	case base_GLUT_SOLID_CUBE:
	//		glutSolidCube(1);
	//		break;

	//	case base_GLUT_WIRE_CONE:
	//	case base_GLUT_SOLID_CONE:
	//		glutSolidCone(1, 1, NUM_SLICES, NUM_STACKS);
	//		break;

	//	case base_GLUT_WIRE_TORUS:
	//	case base_GLUT_SOLID_TORUS:
	//		glutSolidTorus(1, 0.5, NUM_SIDES, NUM_RINGS);
	//		break;

	//	case base_GLUT_WIRE_TEAPOT:
	//	case base_GLUT_SOLID_TEAPOT:
	//		glutSolidTeapot(1);
	//		break;

	//	case base_GLU_SPHERE_LINE:
	//	case base_GLU_SPHERE_FILL:
	//	 	gluQuadricDrawStyle(ptrToQuadricInfo, GLU_FILL);
	//		gluSphere(ptrToQuadricInfo, 1, NUM_SLICES, NUM_STACKS);
	//		break;

	//	case base_GLU_CYLINDER_LINE:
	//	case base_GLU_CYLINDER_FILL:
	//	 	gluQuadricDrawStyle(ptrToQuadricInfo, GLU_FILL);
	//		gluCylinder(ptrToQuadricInfo,
	//					2,1,1, NUM_SLICES, NUM_STACKS);
	//		break;

	//	case base_GLU_DISK_LINE:
	//	case base_GLU_DISK_FILL:
	//	 	gluQuadricDrawStyle(ptrToQuadricInfo, GLU_FILL);
	//		gluDisk(ptrToQuadricInfo,
	//					1, 1.2, NUM_SIDES, NUM_RINGS);
	//		break;

	//	case base_GLU_PARTIALDISK_LINE:
	//	case base_GLU_PARTIALDISK_FILL:
	//	 	gluQuadricDrawStyle(ptrToQuadricInfo, GLU_FILL);
	//		gluPartialDisk(ptrToQuadricInfo,
	//					1, 1.2, NUM_SIDES, NUM_RINGS, 0, 30);
	//		break;
	//	default:
	//		glutWireCube(1);
	//		break;

	//}
//}

void geometricObject::saveToFile_PrototypeInOpenGL(ofstream & outfile)
{
	outfile << endl 
			<< ' ';

	switch(prototype)
	{
		case base_GLUT_WIRE_SPHERE:
			outfile << "glutWireSphere(1, NUM_SLICES, NUM_STACKS); " << endl;
			break;
		case base_GLUT_SOLID_SPHERE:
			outfile << "glutSolidSphere(1, NUM_SLICES, NUM_STACKS);" << endl;
			break;
		case base_GLUT_WIRE_CUBE:
			outfile << "glutWireCube(1);" << endl;
			break;
		case base_GLUT_SOLID_CUBE:
			outfile << "glutSolidCube(1);" << endl;
			break;
		case base_GLUT_WIRE_CONE:
			outfile << "glutWireCone(1, 1, NUM_SLICES, NUM_STACKS);" << endl;
			break;
		case base_GLUT_SOLID_CONE:
			outfile << "glutSolidCone(1, 1, NUM_SLICES, NUM_STACKS);" << endl;
			break;
		case base_GLUT_WIRE_TORUS:
			outfile << "glutWireTorus(1, 0.5, NUM_SIDES, NUM_RINGS);" << endl;
			break;
		case base_GLUT_SOLID_TORUS:
			outfile << "glutSolidTorus(1, 0.5, NUM_SIDES, NUM_RINGS);" << endl;
			break;
		case base_GLUT_WIRE_TEAPOT:
			outfile << "glutWireTeapot(1);" << endl;
			break;
		case base_GLUT_SOLID_TEAPOT:
			outfile << "glutSolidTeapot(1);" << endl;
			break;

		case base_GLU_SPHERE_LINE:
		 	outfile << "gluQuadricDrawStyle(ptrToQuadricInfo, GLU_LINE);" << endl;
			outfile << "gluSphere(ptrToQuadricInfo, 1, NUM_SLICES, NUM_STACKS);" << endl;
			break;
		case base_GLU_SPHERE_FILL:
		 	outfile << "gluQuadricDrawStyle(ptrToQuadricInfo, GLU_FILL);" << endl;
			outfile << "gluSphere(ptrToQuadricInfo, 1, NUM_SLICES, NUM_STACKS);"<< endl;
			break;
		case base_GLU_CYLINDER_LINE:
		 	outfile << "gluQuadricDrawStyle(ptrToQuadricInfo, GLU_LINE);" << endl;
			outfile << "gluCylinder(ptrToQuadricInfo," << endl
					<< "2,1,1, NUM_SLICES, NUM_STACKS);" << endl;
			break;
		case base_GLU_CYLINDER_FILL:
		 	outfile << "gluQuadricDrawStyle(ptrToQuadricInfo, GLU_FILL);" << endl;
			outfile << "gluCylinder(ptrToQuadricInfo," << endl
					<< "2,1,1, NUM_SLICES, NUM_STACKS);" << endl;
			break;
		case base_GLU_DISK_LINE:
		 	outfile << "gluQuadricDrawStyle(ptrToQuadricInfo, GLU_LINE);" << endl;
			outfile << "gluDisk(ptrToQuadricInfo," << endl
					<< "1, 1.2, NUM_SIDES, NUM_RINGS);" << endl;
			break;
		case base_GLU_DISK_FILL:
		 	outfile << "gluQuadricDrawStyle(ptrToQuadricInfo, GLU_FILL);" << endl;
			outfile << "gluDisk(ptrToQuadricInfo," << endl
					<< "1, 1.2, NUM_SIDES, NUM_RINGS);" << endl;
			break;
		case base_GLU_PARTIALDISK_LINE:
		 	outfile << "gluQuadricDrawStyle(ptrToQuadricInfo, GLU_LINE);" << endl;
			outfile << "gluPartialDisk(ptrToQuadricInfo, " << endl
					<< "1, 1.2, NUM_SIDES, NUM_RINGS, 0, 30);" << endl;
			break;
		case base_GLU_PARTIALDISK_FILL:
		 	outfile << "gluQuadricDrawStyle(ptrToQuadricInfo, GLU_FILL);";
			outfile << "gluPartialDisk(ptrToQuadricInfo,"
					<< "1, 1.2, NUM_SIDES, NUM_RINGS, 0, 30);"
					<< endl;
			break;
		default:
			outfile << "glutWireCube(1);" << endl;
			break;
	}
	outfile << endl;
}

void geometricObject::saveToFile_ObjectInfo(ofstream & outfile)
{
  outfile 
	<< prototype << endl
		<< endl
	<<	color[X] << endl
	<<	color[Y] << endl
	<<	color[Z] << endl
		<< endl		
	<<  coordinate[X]	<< endl
	<<  coordinate[Y]	<< endl
	<<  coordinate[Z]	<< endl
		<< endl
	<<	rotate[X]	<< endl
	<<	rotate[Y]	<< endl
	<<	rotate[Z]	<< endl
		<< endl
	<<	scale[X]	<< endl
	<<	scale[Y]	<< endl
	<<	scale[Z]	<< endl
	<< endl;
}

void geometricObject::loadFromFile_ObjectInfo(ifstream & infile)
{
 int intPrototype;
 infile >> intPrototype;
 prototype = (MODEL_PROTOTYPE) intPrototype;

 infile
	>>	color[X] 
	>>	color[Y] 
	>>	color[Z] 
				
	>>  coordinate[X]	
	>>  coordinate[Y]	
	>>  coordinate[Z]	
		
	>>	rotate[X]	
	>>	rotate[Y]	
	>>	rotate[Z]	
		
	>>	scale[X]	
	>>	scale[Y]	
	>>	scale[Z];

	cout 
	<< prototype << endl
		<< endl
	<<	color[X] << endl
	<<	color[Y] << endl
	<<	color[Z] << endl
		<< endl		
	<<  coordinate[X]	<< endl
	<<  coordinate[Y]	<< endl
	<<  coordinate[Z]	<< endl
		<< endl
	<<	rotate[X]	<< endl
	<<	rotate[Y]	<< endl
	<<	rotate[Z]	<< endl
		<< endl
	<<	scale[X]	<< endl
	<<	scale[Y]	<< endl
	<<	scale[Z]	<< endl
	<< "********* finish one object **********" << endl
	<< endl;
}

