#ifndef GEOMETRI_COBJECT_H
#define GEOMETRI_COBJECT_H

enum XYZ_AXIS {X=0, Y=1, Z=2};
enum COLOR_AXIS {R=0, G=1, B=2, A=2};
enum MODEL_PROTOTYPE 
{
	base_GLUT_WIRE_SPHERE = 0,	base_GLUT_SOLID_SPHERE,
	base_GLUT_WIRE_CUBE,		base_GLUT_SOLID_CUBE,
	base_GLUT_WIRE_CONE,		base_GLUT_SOLID_CONE,
	base_GLUT_WIRE_TORUS,		base_GLUT_SOLID_TORUS,
	base_GLUT_WIRE_TEAPOT,		base_GLUT_SOLID_TEAPOT,

	base_GLU_SPHERE_LINE,		base_GLU_SPHERE_FILL,
	base_GLU_CYLINDER_LINE,		base_GLU_CYLINDER_FILL,
	base_GLU_DISK_LINE,			base_GLU_DISK_FILL,
	base_GLU_PARTIALDISK_LINE,	base_GLU_PARTIALDISK_FILL
};

#include <string>
#include <iostream>
#include <fstream>
using namespace std;

class geometricObject
{  
   public:

	geometricObject();
	void moveAlongAxis(XYZ_AXIS axis, float units);
	void rotateAlongAxis(XYZ_AXIS axis, float degrees);
	void scaleAlongAxis(XYZ_AXIS axis, float fraction);
	void scaleColorAlongAxis(COLOR_AXIS axis, float fraction);

	void setPrototype(int aPrototype)
		{prototype = (MODEL_PROTOTYPE) aPrototype; };

	bool isModeledInWirePrototype();

	bool isModeledInSolidPrototype();


	void setOpenGLColorAndMaterialProperties();
	void doOpenGLModelTransformations();

	void renderPrototypeInOpenGL();

	void renderCursorInOpenGL();

	void rendeOriginalRefereceFrameOfPrototype();

	void setObjectCoordinates(float x, float y, float z)
	{	coordinate[X] = x; coordinate[Y] = y; coordinate[Z] = z; };

	void getObjectCoordinates(float &x, float &y, float &z)
	{	x=coordinate[X]; y=coordinate[Y]; z=coordinate[Z]; };

	void displayInfo();
	
	void saveToFile_SetOpenGLColor(ofstream & outfile);

	void saveToFile_OpenGLModelTransformations(ofstream & outfile);

	void saveToFile_PrototypeInOpenGL(ofstream & outfile);

	void saveToFile_ObjectInfo(ofstream & outfile);

	void loadFromFile_ObjectInfo(ifstream & infile);


  private:
	MODEL_PROTOTYPE prototype;

	float color[4];

	float coordinate[3];
	//The cener location of the object

	float rotate[3];

	float scale[3];

	//Thefollowing are static constants defining the precision of
	//	drawing standard glut and dlut 3D geometric objects
	//	in renderPrototypeInOpenGL() and renderCursorInOpenGL()
	static const int NUM_SLICES = 20;
	static const int NUM_STACKS = 20;
	static const int NUM_SIDES = 20;
	static const int NUM_RINGS = 20;

};

#endif
