#include "interfacedScene.h"
#include "myGlutCallbackFunctions.h"
#include <cstring>
//using namespace myGlutCallbackFunctions;

//*****************************************************************
//Tips on the cuurnet user interface mode
//*****************************************************************
const char * TIPS_INTERFACE_INPUT_MODE[]=
				{	"On Selected OBJECT:     SCALE it along PROTOTYPE Axes x,y,z",
					"On Selected OBJECT:     ROTATE it along Axes X,Y,Z",
					"On Selected OBJECT:     MOVE it along Axes X,Y,Z",
					"On Selected OBJECT:     CHANGE its COLOR along RGB(XYZ)Axes",
					"On Selected CAMERA:     MOVE it around FOCUS by mouse or along Axes X,Y,Z by keybord", 
					"On Selected CAMERA:     FLIP it to the OTHER SIDE of an AXIS",
					"On Selected CAMERA:     MOVE its FOCUS (LOOKED-AT POINT) along Axes X,Y,Z",
					"On Selected CAMERA:     ZOOM IN or ZOOM OUT", 
					"FLIGHT SIMULATION :     FLY around and SEE the Scene"
				};

//********************************************************
//Tips on the properties of cameras
//********************************************************
const char * TIPS_CAMERA[]=
				{	"CURRENT CAMERA: Camera A, using glOtho",
					"CURRENT CAMERA: Camera B, using glOtho",
					"CURRENT CAMERA: Camera C, using glPerspective",
					"CURRENT CAMERA: Camera D, using glPerspective"
				};

//*************************************************************************
// Initially there is no scene objects created
//*************************************************************************
int interfacedScene::totalNumberOfSceneObjects = 0;

//Constructor
interfacedScene::interfacedScene(char * mySceneTitle)
{	++interfacedScene::totalNumberOfSceneObjects;
	viewCount = 1;

	if (mySceneTitle != NULL && strlen(mySceneTitle) < 25 )
		strcpy(sceneTitle, mySceneTitle);
	else
	{	strcpy(sceneTitle, "Scene");
		nonnegativeNumToCharsInArray(	totalNumberOfSceneObjects,
										sceneTitle + strlen(sceneTitle), 
										25 - strlen(sceneTitle)
									);
	}

	vectorOfAttachedWindows.clear();

	isReady_SELECT_PROTOTYPE_LIST_SUBSUBMENU = false;
	isReady_SELECT_OBJECT_LIST_SUBSUBMENU = false;
	isReady_DELETE_OBJECT_LIST_SUBSUBMENU = false;
	isReady_SELECT_CAMERA_LIST_SUBSUBMENU = false;

	isReady_INFORMATION_SUBMENU = false;
	isReady_OBJECTS_SUBMENU = false;
	isReady_CAMERAS_SUBMENU = false;

	isReady_MAIN_MENU = false;
}



//Create a new window with the specified window titl2
//Attach the scene to this new window, implicitly select
//	one of the cameras and the object 0 to view the scene
bool interfacedScene::createWindowAttachScene(char * windowTitle)
{	
	int windowID;
	static const int objectID = 0;
	static int cameraID = 0;
	static int interfaceModeID = 0;
	
	//Make sure we have a valid current window. If not, create a new one

	int xPos = 50, yPos = 50;
	if (glutGetWindow() > 0)
	{
		//Slightly scatter the new window position from the current window
		//to avoid complete overlap of windows
		xPos = glutGet(GLUT_WINDOW_X) + 40*( rand()%4 );
		yPos = glutGet(GLUT_WINDOW_Y) + 40*( rand()%4 );
	}

	//Create the new window
	glutInitWindowSize(DEFAULT_WINDOW_WIDTH, DEFAULT_WINDOW_HEIGHT);
	if (windowTitle != NULL && strlen(windowTitle) < 25 )
		windowID = glutCreateWindow( windowTitle );
	else
	{	char title[50];
		//Set up the title as "Scene #1 View #2", 
		//	where #1 and #2 are scene's ID and view's ID respectively
		strcpy(title, sceneTitle);
		strcat(title, "  View ");
		nonnegativeNumToCharsInArray( viewCount, title+strlen(title), 50-strlen(title));
		windowID = glutCreateWindow( title );
	}

	glutPositionWindow(xPos, yPos );


	//This scane is attached to one moew window
	vectorOfAttachedWindows.push_back(windowID);

	//Set this window as the current window
	glutSetWindow( windowID);

	//This scane is attached to one moew window
	vectorOfAttachedWindows.push_back(windowID);

	// set up all the callback functions
		glEnable(GL_DEPTH_TEST);
		glutDisplayFunc(myGlutCallbackFunctions::display);
		glutReshapeFunc(myGlutCallbackFunctions::reshape);
		glutIdleFunc(myGlutCallbackFunctions::idle);
		glutMouseFunc(myGlutCallbackFunctions::mouse);
		glutMotionFunc(myGlutCallbackFunctions::mouseMotion);
		glutKeyboardFunc(myGlutCallbackFunctions::keyboard);

	//Select object 0, a camera, and an inpu mode for this particular window
	scene::selectCamera( cameraID );
	scene::selectObject( objectID );
	currentInterfaceInputMode[windowID] 
		= (INTERFACE_INPUT_MODE) ( interfaceModeID );

	cameraID = (cameraID + 1 ) % scene::totalNumOfCameras();
	interfaceModeID = (interfaceModeID + 1)% get_NUM_INTREFACE_INPUT_MODES();

	//Set this scene as the current scene for the
	//	global call-back functions defined in myGlutCallbackFunctions
	myGlutCallbackFunctions::attachSceneToWindow(*this, windowID);
	++viewCount;

	//set up the menus and their callback functions
	setup_ENTIRE_MENU_HIERARCHY();

	//Set up lighting of the scene
	initializeSceneLightingAndMaterialProperties();

	return true;
}


//Attach the scene to an exisiting window of a specific ID, implicitly select
//	one of the cameras and the object 0 to view the scene
bool interfacedScene::attachToWindow(int windowID, char * windowTitle )
{	static const int objectID = 0;
	static int cameraID = 0;
	static int interfaceModeID = 0;
	
	//Make sure we have a valid current window. If not, create a new one
	if ( windowID <= 0 )
		return false;

	//Set this window as the current window
	glutSetWindow( windowID);

	//Set the title of the window
	if (windowTitle != NULL && strlen(windowTitle) < 25 )
		glutSetWindowTitle( windowTitle );

	//This scane is attached to one moew window
	vectorOfAttachedWindows.push_back(windowID);

	//Set up all the callback functions
		glEnable(GL_DEPTH_TEST);
		glutDisplayFunc(myGlutCallbackFunctions::display);
		glutReshapeFunc(myGlutCallbackFunctions::reshape);
		glutIdleFunc(myGlutCallbackFunctions::idle);
		glutMouseFunc(myGlutCallbackFunctions::mouse);
		glutMotionFunc(myGlutCallbackFunctions::mouseMotion);
		glutKeyboardFunc(myGlutCallbackFunctions::keyboard);

	//Select object 0, a camera, and an inpu mode for this particular window
	scene::selectCamera( cameraID );
	scene::selectObject( objectID );
	currentInterfaceInputMode[windowID] 
		= (INTERFACE_INPUT_MODE) ( interfaceModeID );

	cameraID = (cameraID + 1 ) % scene::totalNumOfCameras();
	interfaceModeID = (interfaceModeID + 1)% get_NUM_INTREFACE_INPUT_MODES();

	//Set this scene as the current scene for the
	//	global call-back functions defined in myGlutCallbackFunctions
	myGlutCallbackFunctions::attachSceneToWindow(*this, windowID);
	++viewCount;


	//set up the menus and their callback functions
	setup_ENTIRE_MENU_HIERARCHY();

	//Set up the lighting of the scene
	initializeSceneLightingAndMaterialProperties();

	return true;
}




//Attach the scene to the current window, also
//	speifying the camera and the object selected to view the scene
//	and the user interface input mode
bool interfacedScene::attachToWindow(int windowID, int cameraID, int objectID, 
									 INTERFACE_INPUT_MODE interfaceMode,
									 char * windowTitle )
{
	//Make sure we have a valid current window
	if ( windowID <= 0 )
		return false;

	//Set this window as the current window
	glutSetWindow( windowID);

	//Set the title of the window
	if (windowTitle != NULL && strlen(windowTitle) < 25 )
		glutSetWindowTitle( windowTitle );
	else
		glutSetWindowTitle( sceneTitle );

	//This scane is attached to one moew window
	vectorOfAttachedWindows.push_back(windowID);

	//Set up all the callback functions
		glutDisplayFunc(myGlutCallbackFunctions::display);
		glutReshapeFunc(myGlutCallbackFunctions::reshape);
		glutIdleFunc(myGlutCallbackFunctions::idle);
		glutMouseFunc(myGlutCallbackFunctions::mouse);
		glutMotionFunc(myGlutCallbackFunctions::mouseMotion);
		glutKeyboardFunc(myGlutCallbackFunctions::keyboard);

	//Select a camera and an object for this particular window
	scene::selectCamera(	cameraID  );
	scene::selectObject( 	objectID  );
	currentInterfaceInputMode[windowID] = interfaceMode;

	//Set this scene as the current scene for the
	//	global call-back functions defined in myGlutCallbackFunctions
	myGlutCallbackFunctions::attachSceneToWindow(*this, windowID);
	++viewCount;

	//set up the menus and their callback functions
	setup_ENTIRE_MENU_HIERARCHY();

	//Set up the lighting of the scene
	initializeSceneLightingAndMaterialProperties();


	return true;
}


//Detach the scene from the current window, also
bool interfacedScene::detachFromWindow(int windowID)
{	myGlutCallbackFunctions::detachSceneFromWindow(*this, windowID);
	//Detach the association in the glut callback functions

	detachSelectedObjectAndCameraFromWindow(windowID);
	//Detach the selected object and selected camera from the window

	currentInterfaceInputMode.erase(windowID);
	//Detach the selected user interface mode from the window

	//Remove the window with windowID from vectorOfAttachedWindows
	for (vector<int>::iterator itr= vectorOfAttachedWindows.begin();
		 itr != vectorOfAttachedWindows.end();
		 ++itr
		 )
	{
		if ( (*itr) == windowID )
		{	
			vectorOfAttachedWindows.erase(itr);
			return true;
		}
	}

	return true;
}


INTERFACE_INPUT_MODE interfacedScene::getCurrentInterfaceInputMode()
		{	return currentInterfaceInputMode[ glutGetWindow() ];	
		}

//set the current interface mode
void interfacedScene::setCurrentInterfaceInputMode(INTERFACE_INPUT_MODE newMode)
		{	currentInterfaceInputMode[ glutGetWindow() ] = newMode;	}

//select the next interface input mode
void interfacedScene::selectNextInterfaceInputMode()
{

	currentInterfaceInputMode[glutGetWindow()] = 
	(INTERFACE_INPUT_MODE)
		( (currentInterfaceInputMode[glutGetWindow()] + 1) 
			% NUM_INTREFACE_INPUT_MODES
		);


	if ( currentInterfaceInputMode[glutGetWindow()] 
			==  TO_DO_FLIGHT_SIMULATION_USING_THIS_SELECTED_CAMERA
	   )
		currentInterfaceInputMode[glutGetWindow()] = 
		(INTERFACE_INPUT_MODE)
			( (currentInterfaceInputMode[glutGetWindow()] + 1) 
				% NUM_INTREFACE_INPUT_MODES
			);

}

//*********************************************************************
///<summary>
/// The following are member functions to set up all kinds of menus...
///</summary>
//**********************************************************************

bool interfacedScene::setup_SELECT_PROTOTYPE_LIST_SUBSUBMENU()
{
	if ( isReady_SELECT_PROTOTYPE_LIST_SUBSUBMENU )
		return true; //Already set up

	id_SELECT_PROTOTYPE_LIST_SUBSUBMENU = 
		glutCreateMenu( myGlutCallbackFunctions::handle_SELECT_PROTOTYPE_LIST_SUBSUBMENU );

			glutAddMenuEntry("base_GLUT_WIRE_SPHERE",	base_GLUT_WIRE_SPHERE);
			glutAddMenuEntry("base_GLUT_WIRE_CUBE",		base_GLUT_WIRE_CUBE);
			glutAddMenuEntry("base_GLUT_WIRE_CONE",		base_GLUT_WIRE_CONE);
			glutAddMenuEntry("base_GLUT_WIRE_TORUS",	base_GLUT_WIRE_TORUS);
			glutAddMenuEntry("base_GLUT_WIRE_TEAPOT",	base_GLUT_WIRE_TEAPOT);

			glutAddMenuEntry("base_GLUT_SOLID_SPHERE",	base_GLUT_SOLID_SPHERE);
			glutAddMenuEntry("base_GLUT_SOLID_CUBE",	base_GLUT_SOLID_CUBE);
			glutAddMenuEntry("base_GLUT_SOLID_CONE",	base_GLUT_SOLID_CONE);
			glutAddMenuEntry("base_GLUT_SOLID_TORUS",	base_GLUT_SOLID_TORUS);
			glutAddMenuEntry("base_GLUT_SOLID_TEAPOT",	base_GLUT_SOLID_TEAPOT);

			glutAddMenuEntry("base_GLU_SPHERE_LINE",		base_GLU_SPHERE_LINE);
			glutAddMenuEntry("base_GLU_CYLINDER_LINE",		base_GLU_CYLINDER_LINE);
			glutAddMenuEntry("base_GLU_DISK_LINE",			base_GLU_DISK_LINE);
			glutAddMenuEntry("base_GLU_PARTIALDISK_LINE",	base_GLU_PARTIALDISK_LINE);

			glutAddMenuEntry("base_GLU_SPHERE_FILL",		base_GLU_SPHERE_FILL);
			glutAddMenuEntry("base_GLU_CYLINDER_FILL",		base_GLU_CYLINDER_FILL);
			glutAddMenuEntry("base_GLU_DISK_FILL",			base_GLU_DISK_FILL);
			glutAddMenuEntry("base_GLU_PARTIALDISK_FILL",	base_GLU_PARTIALDISK_FILL);		

	isReady_SELECT_PROTOTYPE_LIST_SUBSUBMENU = true;
	return true;
}

bool interfacedScene::setup_SELECT_OBJECT_LIST_SUBSUBMENU()
{	if ( isReady_SELECT_OBJECT_LIST_SUBSUBMENU )
		return true; //Already set up

	id_SELECT_OBJECT_LIST_SUBSUBMENU 
		= glutCreateMenu( myGlutCallbackFunctions::handle_SELECT_OBJECT_LIST_SUBSUBMENU );

		char tipsOnSomeObject[30];
		for (int objectID=0; objectID<totalNumOfObjects(); ++objectID)
		{
			setupTipsOnSomeObject(objectID, tipsOnSomeObject, 30);
			glutAddMenuEntry(tipsOnSomeObject, objectID);
		}


	isReady_SELECT_OBJECT_LIST_SUBSUBMENU = true;
	return true;

}

bool interfacedScene::setup_DELETE_OBJECT_LIST_SUBSUBMENU()
{	if ( isReady_DELETE_OBJECT_LIST_SUBSUBMENU )
		return true; //Already set up

	id_DELETE_OBJECT_LIST_SUBSUBMENU
		= glutCreateMenu( myGlutCallbackFunctions::handle_DELETE_OBJECT_LIST_SUBSUBMENU );
		char tipsOnSomeObject[30];
		for (int objectID=0; objectID<totalNumOfObjects(); ++objectID)
		{
			setupTipsOnSomeObject(objectID, tipsOnSomeObject, 30);
			glutAddMenuEntry(tipsOnSomeObject, objectID);
		}

	isReady_DELETE_OBJECT_LIST_SUBSUBMENU = true;
	return true;
}


bool interfacedScene::setup_SELECT_CAMERA_LIST_SUBSUBMENU()
{	if ( isReady_SELECT_CAMERA_LIST_SUBSUBMENU )
		return true; //Already set up

	id_SELECT_CAMERA_LIST_SUBSUBMENU
		= glutCreateMenu( myGlutCallbackFunctions::handle_SELECT_CAMERA_LIST_SUBSUBMENU );
			glutAddMenuEntry("CAMERA A, using glOtho", CAMERA_A);
			glutAddMenuEntry("CAMERA B, using glOtho", CAMERA_B);
			glutAddMenuEntry("CAMERA C, using glPerspective", CAMERA_C);
			glutAddMenuEntry("CAMERA D, using glPerspective", CAMERA_D);

	isReady_SELECT_CAMERA_LIST_SUBSUBMENU = true;
	return true;
}

bool interfacedScene::setup_INFORMATION_SUBMENU()
{	if ( isReady_INFORMATION_SUBMENU )
		return true; //Already set up

	id_INFORMATION_SUBMENU 
		= glutCreateMenu( myGlutCallbackFunctions::handle_INFO_SUBMENU );
			glutAddMenuEntry("SNAPSHOT of SCENE: SAVE as OpenGL commands in SNAPSHOT_VIEW*_yourSceneName.cpp ", 
							SAVE_CURRENT_SNAPSHOT_AS_OPENGL_COMMANDS);
			glutAddMenuEntry("SAVE SCENE: OBJECTS/CAMERAS info saved to SCENE_INFO_yourSceneName.txt", 
							SAVE_SCENE_CAMERAS_OBJECTS_INFO);
			glutAddMenuEntry("LOAD SCENE: OBJECTS/CAMERAS info loaded from SCENE_INFO.txt", 
							LOAD_SCENE_CAMERAS_OBJECTS_INFO);
			glutAddMenuEntry("CREATE a NEW VIEW of SCENE", 
							CREATE_NEW_VIEW_OF_SCENE);
			glutAddMenuEntry("DELETE an EXISTING VIEW of SCENE", 
							DELETE_EXISTING_VIEW_OF_SCENE);

	isReady_INFORMATION_SUBMENU = true;
	return true;
}


bool interfacedScene::setup_OBJECTS_SUBMENU()
{	if ( isReady_OBJECTS_SUBMENU )
		return true; //Already set up

	if (!isReady_SELECT_OBJECT_LIST_SUBSUBMENU			||
		!isReady_SELECT_PROTOTYPE_LIST_SUBSUBMENU		||
		!isReady_DELETE_OBJECT_LIST_SUBSUBMENU
	   )
	{	cout << "Some subsubmenus of the OBJECTS submenu not ready yet" << endl;
		return false;
	}

	id_OBJECTS_SUBMENU 
		= glutCreateMenu( myGlutCallbackFunctions::handle_OBJECTS_SUBMENU );
			glutAddMenuEntry("CREATE and SELECT a new OBJECT", CREATE_AND_SELECT_A_NEW_OBJECT);
			glutAddMenuEntry("DUPLICATE this selected OBJECT", DUPLICATE_THIS_SELECTED_OBJECT);
			glutAddSubMenu("SELECT another OBJECT instead", id_SELECT_OBJECT_LIST_SUBSUBMENU);
			glutAddSubMenu("DELETE an exisiting OBJECT", id_DELETE_OBJECT_LIST_SUBSUBMENU);
			glutAddMenuEntry("________________________", CURRENT_SELECTED_OBJECT);
			glutAddSubMenu("PICK a PROTOTYPE model for this selected OBJECT", id_SELECT_PROTOTYPE_LIST_SUBSUBMENU);
			glutAddMenuEntry("________________________", CURRENT_SELECTED_OBJECT);
			glutAddMenuEntry("SCALE this selected OBJECT", SCALE_THIS_SELECTED_OBJECT);
			glutAddMenuEntry("ROTATE this selected OBJECT", ROTATE_THIS_SELECTED_OBJECT);
			glutAddMenuEntry("MOVE this selected OBJECT", MOVE_THIS_SELECTED_OBJECT);
			glutAddMenuEntry("CHANGE the COLOR of the selected OBJECT", CHANGE_THE_COLOR_OF_THIS_SELECTED_OBJECT);

	isReady_OBJECTS_SUBMENU = true;
	return true;
}

bool interfacedScene::setup_CAMERAS_SUBMENU()
{	if ( isReady_CAMERAS_SUBMENU )
		return true; //Already set up

	if (!isReady_SELECT_CAMERA_LIST_SUBSUBMENU	)
	{	cout << "SELECT_CAMERA_LIST_SUBSUBMENU of the CAMERAS submenu not ready yet" << endl;
		return false;
	}

	id_CAMERAS_SUBMENU 
		= glutCreateMenu( myGlutCallbackFunctions::handle_CAMERAS_SUBMENU );
			glutAddSubMenu("SELECT another CAMERA instead", id_SELECT_CAMERA_LIST_SUBSUBMENU);
			glutAddMenuEntry("______________________________________", CURRENT_SELECTED_CAMERA);
			glutAddMenuEntry("FLIP the selected CAMERA to the OTHER SIDE of an AXIS", 
								FLIP_CAMERA_TO_OTHER_SIDE_OF_AXIS);
			glutAddMenuEntry("MOVE the selected CAMERA", MOVE_THIS_SELECTED_CAMERA);
			glutAddMenuEntry("MOVE the FOCUS (LOOKED at POINT) of this CAMERA", 
								MOVE_LOOKED_AT_POINT_OF_THIS_SELECTED_CAMERA);
			glutAddMenuEntry("ZOOM IN or ZOOM OUT to VIEW the SCENE", 
							  ZOOM_IN_ZOOM_OUT);
			glutAddMenuEntry("_________________", CURRENT_SELECTED_CAMERA);
			glutAddMenuEntry("KEEP LOOKING at the current LOOKED-AT POINT", KEEP_LOOKING_AT_THIS_LOOKED_AT_POINT);
			glutAddMenuEntry("KEEP LOOKING along CURRENT DIRECTION", KEEP_LOOKING_ALONG_THIS_DIRECTION);
			glutAddMenuEntry("_________________", CURRENT_SELECTED_CAMERA);
			glutAddMenuEntry("START AUTOMATIC REVOLVING around the LOOKED at POINT", 
							  START_AUTOMATIC_REVOLVING_AROUND_LOOKED_AT_POINT);
			glutAddMenuEntry("STOP AUTOMATIC REVOLVING: let CAMERA RETURN to previous location", 
					STOP_AUTOMATIC_REVOLVING_AROUND_LOOKED_AT_POINT_AND_RETURN_TO_PREVIOUS_LOCATION);
			glutAddMenuEntry("STOP AUTOMATIC REVOLVING: let CAMERA STAY HERE", 
							  STOP_AUTOMATIC_REVOLVING_AROUND_LOOKED_AT_POINT_AND_STAY_HERE);

	isReady_CAMERAS_SUBMENU = true;
	return true;
}


bool interfacedScene::setup_MAIN_MENU()
{	if ( isReady_MAIN_MENU )
		return true; //Already set up

	if (!isReady_OBJECTS_SUBMENU		||
		!isReady_CAMERAS_SUBMENU		||
		!isReady_INFORMATION_SUBMENU
	   )
	{	cout << "Some submenus of the MAIN menu not ready yet" << endl;
		return false;
	}
			
	id_MAIN_MENU = 
		glutCreateMenu( myGlutCallbackFunctions::handle_MAIN_MENU );

			glutAddSubMenu("HANDLE OBJECTS ...", id_OBJECTS_SUBMENU);
			glutAddSubMenu("HANDLE CAMERAS ...", id_CAMERAS_SUBMENU);

			glutAddMenuEntry(
				"______________________________________",
				CURRENT_CAMERA);

			glutAddMenuEntry("START FLIGHT SIMULATION with this CAMERA",	
				START_FLIGHT_SIMULATION_USING_THIS_CAMERA);

			glutAddMenuEntry("STOP FLIGHT SIMULATION and RETURN to original place",
				STOP_FLIGHT_SIMULATION_AND_RETURN_TO_PREVIOUS_LOCATION);

			glutAddMenuEntry("STOP FLIGHT SIMULATION and STAY HERE",	
				STOP_FLIGHT_SIMULATION_AND_STAY_HERE);

			glutAddMenuEntry(
				"______________________________________",
				CURRENT_CAMERA);

			glutAddMenuEntry("Show XYZ axes and the CURSOR",	SHOW_XYZ_AXES);
			glutAddMenuEntry("Hide XYZ axes and the CURSOR",	HIDE_XYZ_AXES);

			glutAddMenuEntry(
				"______________________________________",
				CURRENT_CAMERA);

			glutAddSubMenu("SNAPSHOT / LOAD / SAVE / CREATE or DELETE VIEW / of the SCENE ...", id_INFORMATION_SUBMENU);

		isReady_MAIN_MENU = true;
		return true;
}



bool interfacedScene::setup_ENTIRE_MENU_HIERARCHY()
{
		//***************************************************
		setup_SELECT_PROTOTYPE_LIST_SUBSUBMENU();

		//***************************************************
		setup_SELECT_OBJECT_LIST_SUBSUBMENU();

		//***************************************************
		setup_DELETE_OBJECT_LIST_SUBSUBMENU();

		//***************************************************
		setup_SELECT_CAMERA_LIST_SUBSUBMENU();

		//***************************************************
		setup_INFORMATION_SUBMENU();

		//***************************************************
		setup_OBJECTS_SUBMENU();

		//***************************************************
		setup_CAMERAS_SUBMENU();

		//***************************************************
		bool menuSystemWellDone = false;
		menuSystemWellDone= setup_MAIN_MENU();;

		glutSetMenu(id_MAIN_MENU);
		glutAttachMenu(GLUT_RIGHT_BUTTON);

		return menuSystemWellDone;
}


bool interfacedScene::rebuild_ENTIRE_MENU_HIERARCHY()
{
	//***************************************************
	//	Destroy all the menus
	//***************************************************
		isReady_SELECT_PROTOTYPE_LIST_SUBSUBMENU = false;
		isReady_SELECT_OBJECT_LIST_SUBSUBMENU = false;
		isReady_DELETE_OBJECT_LIST_SUBSUBMENU = false;
		isReady_SELECT_CAMERA_LIST_SUBSUBMENU = false;

		isReady_INFORMATION_SUBMENU = false;
		isReady_OBJECTS_SUBMENU = false;
		isReady_CAMERAS_SUBMENU = false;

		isReady_MAIN_MENU = false;


		glutDestroyMenu(id_SELECT_PROTOTYPE_LIST_SUBSUBMENU);

		glutDestroyMenu(id_SELECT_OBJECT_LIST_SUBSUBMENU);

		glutDestroyMenu(id_DELETE_OBJECT_LIST_SUBSUBMENU);

		glutDestroyMenu(id_SELECT_CAMERA_LIST_SUBSUBMENU);

		glutDestroyMenu(id_INFORMATION_SUBMENU);

		glutDestroyMenu(id_OBJECTS_SUBMENU);

		glutDestroyMenu(id_CAMERAS_SUBMENU);

		glutDestroyMenu(id_MAIN_MENU);


		//***************************************************
		//	set up all the menus all over again
		//***************************************************
		return setup_ENTIRE_MENU_HIERARCHY();
}


//*************************************************************************
// Several auxilliary functions
//*************************************************************************
void interfacedScene::displayMessageOnCurrentInputMode()
{	setInterfaceModeMessage(	TIPS_INTERFACE_INPUT_MODE[currentInterfaceInputMode[ glutGetWindow() ]]  );	};

const char * interfacedScene::getMessageOnCurrentInputMode()
{ return (	TIPS_INTERFACE_INPUT_MODE[currentInterfaceInputMode[ glutGetWindow() ]]  );	};

int interfacedScene::get_id_SELECT_PROTOTYPE_LIST_SUBSUBMENU()
{ return id_SELECT_PROTOTYPE_LIST_SUBSUBMENU;	};

int interfacedScene::get_id_SELECT_OBJECT_LIST_SUBSUBMENU()
{ return id_SELECT_OBJECT_LIST_SUBSUBMENU; };

int interfacedScene::get_id_DELETE_OBJECT_LIST_SUBSUBMENU()
{return id_DELETE_OBJECT_LIST_SUBSUBMENU;	};

int interfacedScene::get_id_SELECT_CAMERA_LIST_SUBSUBMENU()
{return id_SELECT_CAMERA_LIST_SUBSUBMENU;	};

int interfacedScene::get_id_INFORMATION_SUBMENU()
{return id_INFORMATION_SUBMENU;	};

int interfacedScene::get_id_OBJECTS_SUBMENU()
{return id_OBJECTS_SUBMENU;	};

int interfacedScene::get_id_CAMERAS_SUBMENU()
{return id_CAMERAS_SUBMENU;	};

int interfacedScene::get_id_MAIN_MENU()
{return id_MAIN_MENU;	};


//*********************  A useful global function ****************************
//Try to convert a non-negative interger number into a C_style character string
//	in myCahtArray that can hold arraySize characters.
//*********** Used in the two member functions following it *****************
bool nonnegativeNumToCharsInArray(int number, char * myCharArray, int arraySize)
{
	if (number < 0)
	{	myCharArray[0] = '\0';
		return false;
	}

	int temp = number;
	int length;		//Calculate the number of digits required
	for (length = 1; temp > 9; temp =temp/10)
		++length;

	if (length+ 1 > arraySize )
	{	myCharArray[0] = '\0';
		return false;
	}


	myCharArray[length]='\0';


	int index = length-1;
	if (number == 0)
		myCharArray[index]='0';
	else
		for (temp = number; temp > 0; temp = temp / 10)
		{	myCharArray[index] = '0' + (temp % 10);
			--index;
		}

	return true;
}

bool interfacedScene::setupTipsOnCurrentObject(char * tipsOnCurrentObject, int size)
{
	if (size < 20)
		return false;

	strcpy(tipsOnCurrentObject, "CURRENT OBJECT: ");

	char text[30];
	if ( nonnegativeNumToCharsInArray(currentObject(), text,30) == false)
		return false;

	strcat(tipsOnCurrentObject, text);
	cout << tipsOnCurrentObject << endl;
	return true;
}



bool interfacedScene::setupTipsOnNewObject(char * tipsOnNewObject, int size)
{
	if (size < 10)
		return false;

	strcpy(tipsOnNewObject, "OBJECT: ");

	int newObjectID = totalNumOfObjects()-1;
	char text[30];
	if ( nonnegativeNumToCharsInArray(newObjectID, text,30) == false)
		return false;

	strcat(tipsOnNewObject, text);
	cout << tipsOnNewObject << endl;
	return true;
}

bool interfacedScene::setupTipsOnSomeObject(int objectID, char * tipsOnSomeObject, int size)
{
	if (size < 10)
		return false;

	strcpy(tipsOnSomeObject, "OBJECT: ");

	char text[30];
	if ( nonnegativeNumToCharsInArray(objectID, text,30) == false)
		return false;

	strcat(tipsOnSomeObject, text);
	cout << tipsOnSomeObject << endl;
	return true;
}


bool interfacedScene::setupTipsOnCurrentCamera(char * tipsOnCurrentCamera, int size)
{
	if (size < 50)
		return false;
	strcpy(tipsOnCurrentCamera, TIPS_CAMERA[currentCamera()]);
	return true;

}

const char * interfacedScene::getMessageOnCurrentCamera()
{
	return TIPS_CAMERA[currentCamera()];
}


void interfacedScene::keysToActionsMappingHelp()
{  
	cout << "**************   Functionality of Keys **************" << endl
		<< "	X_up:	k " << endl
		<< "	X_down:	j " << endl
		<< "	Y_up:	i " << endl
		<< "	Y_down:	m " << endl
		<< "	Z_up:	l " << endl
		<< "	Z_down:	h " << endl 
		<< endl
		<< "	R_up:	k " << endl
		<< "	R_down:	j " << endl
		<< "	G_up:	i " << endl
		<< "	G_down:	m " << endl
		<< "	B_up:	l " << endl
		<< "	B_down:	h " << endl << endl;

}



