#ifndef INTERFACE_TO_SCENE_H
#define INTERFACE_TO_SCENE_H
#include <map>

#include <GL/glut.h>

#include "scene.h"		
//We derive the interfacedScene class from the scene class

#include "menusAndEntries.h"
//Information of enumeration types, each symbolically enumerates
//	the names of the entries of one menu


//*********************************************************
//	The following glutInterfacedScene class is derived from
//		the scene class to have a user interface with GLUT 
//*********************************************************
class interfacedScene: public scene
{
  private: //private data members

	//A static calss member to keep track of the total number of
	//	 scene objects created so far
	static int totalNumberOfSceneObjects;

	int viewCount;

	//Titile of the scene
	char sceneTitle[25];

	//******************************************************************
	//A vector keeping the IDs of windows attached to by this scene
	//******************************************************************
	typedef int WINDOW_ID;
	vector<WINDOW_ID> vectorOfAttachedWindows;

	//************************************************************************
	//Information of the current user interface input mode for each window
	//	this scene is attached to.
	//For example, currentInterfaceInputMode[windowID] gives you the cuurent
	//	user interface input mode of the window with the ID windowID.
	//In particular, currentInterfaceInputMode[ glutGetWindow() ] tells you 
	//	the cuurent user interface input mode of the current window.
	//************************************************************************
	//typedef int WINDOW_ID;
	map<WINDOW_ID, INTERFACE_INPUT_MODE> currentInterfaceInputMode;


	//Number of different user interface modes available
	static const int NUM_INTREFACE_INPUT_MODES = 9;


	//********************************************************
	//int variables for storing the ID numbers of menus
	//********************************************************
	int id_SELECT_PROTOTYPE_LIST_SUBSUBMENU;
	int id_SELECT_OBJECT_LIST_SUBSUBMENU;
	int id_DELETE_OBJECT_LIST_SUBSUBMENU;
	int id_SELECT_CAMERA_LIST_SUBSUBMENU;

	int id_INFORMATION_SUBMENU;
	int id_OBJECTS_SUBMENU;
	int id_CAMERAS_SUBMENU;

	int id_MAIN_MENU;

	//********************************************************
	//bool variables indicating 
	//	whether a particular menu iscreated already
	//********************************************************
	bool isReady_SELECT_PROTOTYPE_LIST_SUBSUBMENU;
	bool isReady_SELECT_OBJECT_LIST_SUBSUBMENU;
	bool isReady_DELETE_OBJECT_LIST_SUBSUBMENU;
	bool isReady_SELECT_CAMERA_LIST_SUBSUBMENU;

	bool isReady_INFORMATION_SUBMENU;
	bool isReady_OBJECTS_SUBMENU;
	bool isReady_CAMERAS_SUBMENU;

	bool isReady_MAIN_MENU;


 public:

	//Constructor method
	//	mySceneTile is set to NULL if the caller doesn't provide one.
	interfacedScene(char * mySceneTitle = NULL);

	//*********************************************************
	//	Three methods to attach the interfaced scene to a window.  
	//	windowTile is set to NULL if the caller doesn't provide one.
	//*********************************************************
	bool createWindowAttachScene(char * windowTitle = NULL); 
	//Create a new window with the windowTile as title and
	//	attach the scene to the window.

	bool attachToWindow(int windowID, char * windowTitle = NULL);
	bool attachToWindow(int windowID, int cameraID, int objectID, 
						 INTERFACE_INPUT_MODE interfaceMode,
						 char * windowTitle = NULL);


	//Detach the scene from the window with the ID: windowID 
	bool detachFromWindow(int windowID);

	//Get the current interface mode
	INTERFACE_INPUT_MODE getCurrentInterfaceInputMode();
	
	//set the current interface mode
	void setCurrentInterfaceInputMode(INTERFACE_INPUT_MODE newMode);
	
	//Selec the next user interface input mode
	void selectNextInterfaceInputMode();

	void displayMessageOnCurrentInputMode();

	const char * getMessageOnCurrentInputMode();

	const char * getMessageOnCurrentCamera();

	int get_id_SELECT_PROTOTYPE_LIST_SUBSUBMENU();

	int get_id_SELECT_OBJECT_LIST_SUBSUBMENU();

	int get_id_DELETE_OBJECT_LIST_SUBSUBMENU();

	int get_id_SELECT_CAMERA_LIST_SUBSUBMENU();

	int get_id_INFORMATION_SUBMENU();

	int get_id_OBJECTS_SUBMENU();

	int get_id_CAMERAS_SUBMENU();

	int get_id_MAIN_MENU();

	int get_NUM_INTREFACE_INPUT_MODES()
		{ return NUM_INTREFACE_INPUT_MODES;	};

	static const int DEFAULT_WINDOW_WIDTH = 500;
	static const int DEFAULT_WINDOW_HEIGHT = 500;

	//*********************************************************
	//	auxilliary menthods
	//*********************************************************
	bool setupTipsOnCurrentObject(char * tipsOnCurrentObject, int size);

	bool setupTipsOnNewObject(char * tipsOnCurrentObject, int size);

	bool setupTipsOnSomeObject(int objectID, char * tipsOnSomeObject, int size);

	bool setupTipsOnCurrentCamera(char * tipsOnCurrentCamera, int size);

	const char * getSceneTitle()
		{	return sceneTitle;	};

	int getViewCount()
		{ return viewCount;	};

	void keysToActionsMappingHelp();

 private:
	//*********************************************************
	//	Menthods to set up the menus and the entire menu hierarchy
	//*********************************************************
	bool setup_SELECT_PROTOTYPE_LIST_SUBSUBMENU();

	bool setup_SELECT_OBJECT_LIST_SUBSUBMENU();

	bool setup_DELETE_OBJECT_LIST_SUBSUBMENU();

	bool setup_SELECT_CAMERA_LIST_SUBSUBMENU();

	bool setup_INFORMATION_SUBMENU();

	bool setup_OBJECTS_SUBMENU();

	bool setup_CAMERAS_SUBMENU();

	bool setup_MAIN_MENU();

	bool setup_ENTIRE_MENU_HIERARCHY();

 public:
	bool rebuild_ENTIRE_MENU_HIERARCHY();

};

bool nonnegativeNumToCharsInArray(int number, char * myCharArray, int arraySize);
//A useful global function to convert a number into a C-style char array

#endif
