#include "myGlutCallbackFunctions.h"
#include <map>
#include <cmath>
#include <iostream>
using namespace std;

	//*****************************************************************
	// A static class member used to point to the actual scene currently 
	//		attached to the window generating the event.
	// It is used by all the static member callback functions defined here
	//	 to modify the contents of the current scene
	//*****************************************************************
	interfacedScene * myGlutCallbackFunctions::ptrToCurrentScene = NULL;


	//*****************************************************************
	// A static member keeping the information of scenes attached to
	//	differnt windows. 
	// For exmaple, map_WindowID_To_ScenePointer[id] will give you
	//	the pointer pointing to the scene attached to the window 
	//	with id as its window ID
	//*****************************************************************
	typedef int WINDOW_ID;
	map<WINDOW_ID, interfacedScene *> myGlutCallbackFunctions::map_WindowID_To_ScenePointer;


	//*****************************************************************
	// A static member keeping the information of the next window
	//	that we should refresh in the next idle event. 
	//*****************************************************************
	map<WINDOW_ID, interfacedScene *>::iterator 
			myGlutCallbackFunctions::itrNextWindowToSceneEntry 
			= map_WindowID_To_ScenePointer.begin();

	//*****************************************************************
	// A static member function to attach a scene to a window 
	// This scene will then interact through this wondow
	//	 with all the static member GLUT callback functions  
	//	 defined in myGlutCallbackFunctions.cpp 
	//*****************************************************************
	void myGlutCallbackFunctions::
	attachSceneToWindow(interfacedScene & myScene, int windowID)
	{	map_WindowID_To_ScenePointer[windowID] = & myScene;
	}


	void myGlutCallbackFunctions::
	detachSceneFromWindow(interfacedScene & myScene, int windowID)
	{	if (	windowID <= 0 ||
				(	map_WindowID_To_ScenePointer.find(windowID) 
					== map_WindowID_To_ScenePointer.end()
				)
		   )
			return; // myScene is not attached to a valid window

		if (map_WindowID_To_ScenePointer[windowID] != (&myScene) )
			return; // myScene is not attached to this window


		// erase the mapping from windowID to scene pointer
		map_WindowID_To_ScenePointer.erase(windowID);

		//Reset the iterator for refreshing windows
		itrNextWindowToSceneEntry = map_WindowID_To_ScenePointer.begin();

		//Clear the contents of the detached window
		glutSetWindow(windowID);
		glClearColor(0.0, 0.0, 0.0, 1.0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glFlush();

	}


	void myGlutCallbackFunctions::
	initializeGlut(int & argc, char **argv)
	{
		glutInit(&argc, argv);
		glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);

		glClearColor (0.0, 0.0, 0.0, 0.0);
		glShadeModel (GL_SMOOTH);
		glEnable(GL_DEPTH_TEST);

	}




	//Get the pointer to the scene attacked to the current window
	interfacedScene * myGlutCallbackFunctions::getPtrToSceneAttachedToCurrentWindow()
	{
		int ID_CurrentWindow = glutGetWindow();
		if (	( ID_CurrentWindow <= 0 )	 ||
				( map_WindowID_To_ScenePointer.find( ID_CurrentWindow )
					== map_WindowID_To_ScenePointer.end()
				)
		   )
			return NULL; //Not a valid window or no scene attached
		else
			return map_WindowID_To_ScenePointer[ID_CurrentWindow];
	    
	}

	//*********************************************************************
	///<summary>
	/// Section 0.
	/// Consntants used by the static member callback functions to process GUI events 
	///</summary>
	//**********************************************************************

	// enum XYZ_AXIS {X=0, Y=1, Z=2};	//Defined in "geometricObject.h"
	// enum COLOR_AXIS {R=0, G=1, B=2};	//Defined in "geometricObject.h"

	//*************************************************************
	//Mapping keys to actions of increment or decrement
	//along the X,Y Z coordinate axes,  RGB color axes, and 
	//gluPerspective fov angle
	//*************************************************************
	const char myGlutCallbackFunctions::KEY_TO_NO_ACTION	
								= (char) 0; // The ASCII code for NULL character
	const char myGlutCallbackFunctions::KEY_TO_UNKNOWN_ACTION	
								= (char) 0; // The ASCII code for NULL character

	const char myGlutCallbackFunctions::KEY_TO_SELECT_NEXT_OBJECT	=	' ';
	const char myGlutCallbackFunctions::KEY_TO_SELECT_NEXT_CAMERA	=	'\t';
	const char myGlutCallbackFunctions::KEY_TO_SELECT_NEXT_INTERFACE_INPUT_MODE 
								= (char) 13; // The ASCII code for the enter key

	const char myGlutCallbackFunctions::X_up	=	'k';
	const char myGlutCallbackFunctions::X_down = 'j';
	const char myGlutCallbackFunctions::Y_up =	'i';
	const char myGlutCallbackFunctions::Y_down =	'm';
	const char myGlutCallbackFunctions::Z_up =	'l';
	const char myGlutCallbackFunctions::Z_down =	'h';

	const char myGlutCallbackFunctions::R_up	=	'k';
	const char myGlutCallbackFunctions::R_down = 'j';
	const char myGlutCallbackFunctions::G_up =	'i';
	const char myGlutCallbackFunctions::G_down =	'm';
	const char myGlutCallbackFunctions::B_up =	'l';
	const char myGlutCallbackFunctions::B_down =	'h';

	const char myGlutCallbackFunctions::FOV_up1	=	'k';
	const char myGlutCallbackFunctions::FOV_down1 =	'j';
	const char myGlutCallbackFunctions::FOV_up2 =	'i';
	const char myGlutCallbackFunctions::FOV_down2 =	'm';
	const char myGlutCallbackFunctions::FOV_up3 =	'l';
	const char myGlutCallbackFunctions::FOV_down3 =	'h';

	//************************************************************************
	//Definging basic unit of change 
	//along the X,Y Z coordinate axes and RGB color axes
	//*************************************************************
	const float myGlutCallbackFunctions::UNIT_SCALE_IN_XYZ			= (float)0.05;
	const float myGlutCallbackFunctions::UNIT_ROTATE_IN_XYZ			= (float)1.0;
	const float myGlutCallbackFunctions::UNIT_MOVE_IN_XYZ			= (float)0.05;
	const float myGlutCallbackFunctions::UNIT_COLOR_CHANGE_IN_RGB	= (float)0.02;



//*********************************************************************
///<summary>
/// Section 1.
/// static member callback functions to process GUI events from glut such as
///  display, reshape, mouse, keyboard, ...
///</summary>
/**********************************************************************

		//********************************************************
		// Event handler for GLUT display events
		/***********************************************************/
		void myGlutCallbackFunctions::display()
		{	
			ptrToCurrentScene = getPtrToSceneAttachedToCurrentWindow();
			if ( ptrToCurrentScene == NULL )	//No scene attached simply return
				return;

			if (ptrToCurrentScene->getCurrentInterfaceInputMode() == TO_SCALE_THIS_SELECTED_OBJECT)
				ptrToCurrentScene->setToShowOriginalRefereceFrameOfSelectedObject();
			else
				ptrToCurrentScene->setNotToShowOriginalRefereceFrameOfSelectedObject();

			ptrToCurrentScene->displayMessageOnCurrentInputMode();

			ptrToCurrentScene->renderSceneSeenFromCurrentCamera();
		}


		//Trigger an action according to what is the key and what is the current interface mode
		void myGlutCallbackFunctions::
			triggerActionAccordingToInterfaceMode(unsigned char key)
		{	
			ptrToCurrentScene = getPtrToSceneAttachedToCurrentWindow();
			if ( ptrToCurrentScene == NULL )	//No scene attached simply return
				return;

			switch ( ptrToCurrentScene->getCurrentInterfaceInputMode() )
			{
				case TO_SCALE_THIS_SELECTED_OBJECT:
					if (key == X_up)
						ptrToCurrentScene->scaleThisObjectAlongAxis(X,UNIT_SCALE_IN_XYZ);
					if (key == X_down)
						ptrToCurrentScene->scaleThisObjectAlongAxis(X,-UNIT_SCALE_IN_XYZ);
					if (key == Y_up)
						ptrToCurrentScene->scaleThisObjectAlongAxis(Y,UNIT_SCALE_IN_XYZ);
					if (key == Y_down)
						ptrToCurrentScene->scaleThisObjectAlongAxis(Y,-UNIT_SCALE_IN_XYZ);
					if (key == Z_up)
						ptrToCurrentScene->scaleThisObjectAlongAxis(Z,UNIT_SCALE_IN_XYZ);
					if (key == Z_down)
						ptrToCurrentScene->scaleThisObjectAlongAxis(Z,-UNIT_SCALE_IN_XYZ);
					break;

				case TO_ROTATE_THIS_SELECTED_OBJECT:
					if (key == X_up)
						ptrToCurrentScene->rotateThisObjectAlongAxis(X,UNIT_ROTATE_IN_XYZ);
					if (key == X_down)
						ptrToCurrentScene->rotateThisObjectAlongAxis(X,-UNIT_ROTATE_IN_XYZ);
					if (key == Y_up)
						ptrToCurrentScene->rotateThisObjectAlongAxis(Y,UNIT_ROTATE_IN_XYZ);
					if (key == Y_down)
						ptrToCurrentScene->rotateThisObjectAlongAxis(Y,-UNIT_ROTATE_IN_XYZ);
					if (key == Z_up)
						ptrToCurrentScene->rotateThisObjectAlongAxis(Z,UNIT_ROTATE_IN_XYZ);
					if (key == Z_down)
						ptrToCurrentScene->rotateThisObjectAlongAxis(Z,-UNIT_ROTATE_IN_XYZ);
					break;

				case TO_MOVE_THIS_SELECTED_OBJECT:
					if (key == X_up)
						ptrToCurrentScene->moveThisObjectAlongAxis(X,UNIT_MOVE_IN_XYZ);
					if (key == X_down)
						ptrToCurrentScene->moveThisObjectAlongAxis(X,-UNIT_MOVE_IN_XYZ);
					if (key == Y_up)
						ptrToCurrentScene->moveThisObjectAlongAxis(Y,UNIT_MOVE_IN_XYZ);
					if (key == Y_down)
						ptrToCurrentScene->moveThisObjectAlongAxis(Y,-UNIT_MOVE_IN_XYZ);
					if (key == Z_up)
						ptrToCurrentScene->moveThisObjectAlongAxis(Z,UNIT_MOVE_IN_XYZ);
					if (key == Z_down)
						ptrToCurrentScene->moveThisObjectAlongAxis(Z,-UNIT_MOVE_IN_XYZ);
					break;

				case TO_CHANGE_THE_COLOR_OF_THIS_SELECTED_OBJECT:
					if (key == R_up)
						ptrToCurrentScene->scaleThisObjectColorAlongAxis(R,UNIT_COLOR_CHANGE_IN_RGB);
					if (key == R_down)
						ptrToCurrentScene->scaleThisObjectColorAlongAxis(R,-UNIT_COLOR_CHANGE_IN_RGB);
					if (key == G_up)
						ptrToCurrentScene->scaleThisObjectColorAlongAxis(G,UNIT_COLOR_CHANGE_IN_RGB);
					if (key == G_down)
						ptrToCurrentScene->scaleThisObjectColorAlongAxis(G,-UNIT_COLOR_CHANGE_IN_RGB);
					if (key == B_up)
						ptrToCurrentScene->scaleThisObjectColorAlongAxis(B,UNIT_COLOR_CHANGE_IN_RGB);
					if (key == B_down)
						ptrToCurrentScene->scaleThisObjectColorAlongAxis(B,-UNIT_COLOR_CHANGE_IN_RGB);
					break;

				case TO_MOVE_THIS_SELECTED_CAMERA: 
					if (key == X_up)
						ptrToCurrentScene->moveThisCameraAlongAxis(X,UNIT_MOVE_IN_XYZ);
					if (key == X_down)
						ptrToCurrentScene->moveThisCameraAlongAxis(X,-UNIT_MOVE_IN_XYZ);
					if (key == Y_up)
						ptrToCurrentScene->moveThisCameraAlongAxis(Y,UNIT_MOVE_IN_XYZ);
					if (key == Y_down)
						ptrToCurrentScene->moveThisCameraAlongAxis(Y,-UNIT_MOVE_IN_XYZ);
					if (key == Z_up)
						ptrToCurrentScene->moveThisCameraAlongAxis(Z,UNIT_MOVE_IN_XYZ);
					if (key == Z_down)
						ptrToCurrentScene->moveThisCameraAlongAxis(Z,-UNIT_MOVE_IN_XYZ);
					break;

				case TO_FLIP_CAMERA_TO_OTHER_SIDE_OF_AXIS:
					if (key == X_up || key == X_down)
						ptrToCurrentScene->flipThisCameraToOtherSideOfAxis(X);
					if (key == Y_up || key == Y_down)
						ptrToCurrentScene->flipThisCameraToOtherSideOfAxis(Y);
					if (key == Z_up || key == Z_down)
						ptrToCurrentScene->flipThisCameraToOtherSideOfAxis(Z);
					break;

				case TO_MOVE_LOOKED_AT_POINT_OF_THIS_SELECTED_CAMERA: 
					if (key == X_up)
						ptrToCurrentScene->moveLookedAtPointOfThisCameraAlongAxis(X,UNIT_MOVE_IN_XYZ);
					if (key == X_down)
						ptrToCurrentScene->moveLookedAtPointOfThisCameraAlongAxis(X,-UNIT_MOVE_IN_XYZ);
					if (key == Y_up)
						ptrToCurrentScene->moveLookedAtPointOfThisCameraAlongAxis(Y,UNIT_MOVE_IN_XYZ);
					if (key == Y_down)
						ptrToCurrentScene->moveLookedAtPointOfThisCameraAlongAxis(Y,-UNIT_MOVE_IN_XYZ);
					if (key == Z_up)
						ptrToCurrentScene->moveLookedAtPointOfThisCameraAlongAxis(Z,UNIT_MOVE_IN_XYZ);
					if (key == Z_down)
						ptrToCurrentScene->moveLookedAtPointOfThisCameraAlongAxis(Z,-UNIT_MOVE_IN_XYZ);
					break;							

				case TO_ZOOM_IN_ZOOM_OUT: 
					if (key == FOV_up1 || key == FOV_up2 || key == FOV_up3)
						ptrToCurrentScene->zoomOutThisCameraView();
					if (key == FOV_down1 || key == FOV_down2 || key == FOV_down3)
						ptrToCurrentScene->zoomInThisCameraView();

					break;

				default:
					break;
			}

		}


		//********************************************************
		// Event handler for GLUT keyboard events
		/***********************************************************/
		void myGlutCallbackFunctions::keyboard(unsigned char key, int x, int y)
		{
			ptrToCurrentScene = getPtrToSceneAttachedToCurrentWindow();
			if ( ptrToCurrentScene == NULL )	//No scene attached simply return
				return;

			if (key == KEY_TO_SELECT_NEXT_OBJECT)
				ptrToCurrentScene->selectNextObject();

			else if (key == KEY_TO_SELECT_NEXT_CAMERA)
				ptrToCurrentScene->selectNextCamera();

			else if (key == KEY_TO_SELECT_NEXT_INTERFACE_INPUT_MODE)
				ptrToCurrentScene->selectNextInterfaceInputMode();

			else
				triggerActionAccordingToInterfaceMode(key);

			cout << "************ Keyboard Event **************" << endl;
			cout << "Current interface input mode:"  
				<< ptrToCurrentScene->getCurrentInterfaceInputMode() << endl
				<< " This character pressed:" << key << '=' << (int) key << endl
				<< " Selected Object ID=" << ptrToCurrentScene->currentObject() << endl
				<< " selected camera ID= " << ptrToCurrentScene->currentCamera() << endl;
			cout << "New interface mode:" 
				 << ptrToCurrentScene->getCurrentInterfaceInputMode() << endl << endl;

			cout << "*****************************************" << endl << endl;

			glutPostRedisplay();
		}


		//********************************************************
		// Event handler for GLUT reshape events
		/***********************************************************/
		void myGlutCallbackFunctions::reshape(int w, int h)
		{
			ptrToCurrentScene = getPtrToSceneAttachedToCurrentWindow();
			if ( ptrToCurrentScene == NULL )	//No scene attached simply return
				return;

			ptrToCurrentScene->doOpenGLReshape( w, h);		
		}


		//********************************************************
		// Event handler for GLUT idle events
		/***********************************************************/
		void myGlutCallbackFunctions::idle()
		{
			if ( map_WindowID_To_ScenePointer.empty() )
				return; //No Scene attached to any window. Simply return

			//Reset the iterator if we reach the end of the map
			if ( itrNextWindowToSceneEntry  == map_WindowID_To_ScenePointer.end() )
				itrNextWindowToSceneEntry = map_WindowID_To_ScenePointer.begin();

			// itrNextWindowToSceneEntry->first		//this is the next window ID
			// Set it as the current window
			glutSetWindow(	itrNextWindowToSceneEntry->first	);

			// itrNextWindowToSceneEntry->second	//this is the pointer to the scene attached to that window
			ptrToCurrentScene = itrNextWindowToSceneEntry->second;

			//Update the scene attached to the current window if it is a valid scene
			if ( ptrToCurrentScene != NULL )	
				ptrToCurrentScene->updateScene();	

			++itrNextWindowToSceneEntry;
		}



		void myGlutCallbackFunctions::mouse(int button, int state, int x, int y)
		{
			ptrToCurrentScene = getPtrToSceneAttachedToCurrentWindow();
			if ( ptrToCurrentScene == NULL )	//No scene attached simply return
				return;

			//Change the cursor to a crosshair cursor if the left button is pressed
			if ( button == GLUT_LEFT_BUTTON && state == GLUT_DOWN )
				glutSetCursor( GLUT_CURSOR_CROSSHAIR );

			if ( button == GLUT_LEFT_BUTTON && state == GLUT_UP)
				glutSetCursor( GLUT_CURSOR_LEFT_ARROW);

		}




		char myGlutCallbackFunctions::
			mapMouseMotionToAction(double mouseMotionX, double  mouseMotionY)
		{
			//ptrToCurrentScene = getPtrToSceneAttachedToCurrentWindow();
			//if ( ptrToCurrentScene == NULL )	//No scene attached simply return
			//	return KEY_TO_UNKNOWN_ACTION;

			//if	( ptrToCurrentScene->getCurrentInterfaceInputMode() 
			//	  == TO_MOVE_THIS_SELECTED_CAMERA
			//	)
			//{	ptrToCurrentScene->
			//		moveThisCameraAccordingToMouseMotion(mouseMotionX, mouseMotionY);
			//	return KEY_TO_NO_ACTION;
			//}


			double axisX_newX,	axisX_newY,		axisX_newZ;
			double axisY_newX,	axisY_newY,		axisY_newZ;
			double axisZ_newX,	axisZ_newY,		axisZ_newZ;
			double origin_newX, origin_newY,	origin_newZ;

			ptrToCurrentScene->
			projectToScreenCoordinates	(	1,	0,	0,	
											axisX_newX, axisX_newY, axisX_newZ
										);

			ptrToCurrentScene->
			projectToScreenCoordinates	(	0,	1,	0,	
											axisY_newX, axisY_newY, axisY_newZ
										);

			ptrToCurrentScene->
			projectToScreenCoordinates	(	0,	0,	1,	
											axisZ_newX, axisZ_newY, axisZ_newZ
										);

			ptrToCurrentScene->
			projectToScreenCoordinates	(	0,	0,	0,	
											origin_newX, origin_newY, origin_newZ
										);
			
			double axisX_diffX,	axisX_diffY;
			double axisY_diffX,	axisY_diffY;
			double axisZ_diffX,	axisZ_diffY;

			axisX_diffX = axisX_newX - origin_newX;
			axisX_diffY = axisX_newY - origin_newY;

			axisY_diffX = axisY_newX - origin_newX;
			axisY_diffY = axisY_newY - origin_newY;

			axisZ_diffX = axisZ_newX - origin_newX;
			axisZ_diffY = axisZ_newY - origin_newY;


			double	len_MouseMotion, 
					len_ProjectedAxisX,
					len_ProjectedAxisY,
					len_ProjectedAxisZ;

			len_MouseMotion 
				= sqrt( mouseMotionX *  mouseMotionX +  mouseMotionY *  mouseMotionY);

			len_ProjectedAxisX 
				= sqrt(axisX_diffX * axisX_diffX + axisX_diffY * axisX_diffY);

			len_ProjectedAxisY 
				= sqrt(axisY_diffX * axisY_diffX + axisY_diffY * axisY_diffY);

			len_ProjectedAxisZ 
				= sqrt(axisZ_diffX * axisZ_diffX + axisZ_diffY * axisZ_diffY);

			static const double THRESHOLD_LENGTH = 0.001;
			if	(	len_MouseMotion			< THRESHOLD_LENGTH	||
					len_ProjectedAxisX		< THRESHOLD_LENGTH	||
					len_ProjectedAxisY		< THRESHOLD_LENGTH	||
					len_ProjectedAxisZ		< THRESHOLD_LENGTH	
				)
				return KEY_TO_UNKNOWN_ACTION;


			mouseMotionX  /= len_MouseMotion;
			mouseMotionY  /= len_MouseMotion;

			axisX_diffX /= len_ProjectedAxisX;
			axisX_diffY /= len_ProjectedAxisX;

			axisY_diffX /= len_ProjectedAxisY;
			axisY_diffY /= len_ProjectedAxisY;

			axisZ_diffX /= len_ProjectedAxisZ;
			axisZ_diffY /= len_ProjectedAxisZ;


			double thetaX, thetaY, thetaZ;
			thetaX = acos( axisX_diffX * mouseMotionX + axisX_diffY * mouseMotionY);
			thetaY = acos( axisY_diffX * mouseMotionX + axisY_diffY * mouseMotionY);
			thetaZ = acos( axisZ_diffX * mouseMotionX + axisZ_diffY * mouseMotionY);

			cout << endl << endl
				 << "X vector=(" << axisX_diffX << ", " << axisX_diffY << ") " << endl
				 << "Y vector=(" << axisY_diffX << ", " << axisY_diffY << ") " << endl
				 << "Z vector=(" << axisZ_diffX << ", " << axisZ_diffY << ") " << endl;

			cout << " thetaX=" << thetaX*180.0/3.14159 << endl
				 << " thetaY=" << thetaY*180.0/3.14159 << endl 
				 << " thetaZ=" << thetaZ*180.0/3.14159 << endl; 

			static const double THRESHOLD_RADIAN = 0.5;
			static const double PI = 3.14159;

			if (	thetaX			<	THRESHOLD_RADIAN	)
				return X_up;
			if (	( PI - thetaX )	<	THRESHOLD_RADIAN	)
				return X_down;

			if (	thetaY			<	THRESHOLD_RADIAN	)
				return Y_up;
			if (	( PI - thetaY )	<	THRESHOLD_RADIAN	)
				return Y_down;

			if (	thetaZ			<	THRESHOLD_RADIAN	)
				return Z_up;
			if (	( PI - thetaZ )	<	THRESHOLD_RADIAN	)
				return Z_down;

		}



		void myGlutCallbackFunctions::mouseMotion(int x, int y)
		{	static int oldX = -1;
			static int oldY = -1;

			if (oldX == -1 && oldY == -1)
			{	oldX = x;	
				oldY = y;
				return;
			}

			double motionX=0, motionY=0;
				motionX =	x - oldX;
				motionY = -(y - oldY);

			static const double MOTION_LOWER_THRESHOLD = 5.0;
			if ( fabs(motionX) + fabs(motionY) < MOTION_LOWER_THRESHOLD )
				return;

			static const double MOTION_UPPER_THRESHOLD = 45.0;
			if ( fabs(motionX) + fabs(motionY) > MOTION_UPPER_THRESHOLD )
			{
				oldX = x;	
				oldY = y;
				return;
			}

			oldX = x;	
			oldY = y;



			ptrToCurrentScene = getPtrToSceneAttachedToCurrentWindow();
			if ( ptrToCurrentScene == NULL )	//No scene attached simply return
					return;
			else if	( ptrToCurrentScene->getCurrentInterfaceInputMode() 
						== TO_MOVE_THIS_SELECTED_CAMERA
					)
					{	ptrToCurrentScene->
							moveThisCameraAccordingToMouseMotion(motionX, motionY);
					}
			else if	(  ptrToCurrentScene->getCurrentInterfaceInputMode() 
						== TO_DO_FLIGHT_SIMULATION_USING_THIS_SELECTED_CAMERA
					)
					{	ptrToCurrentScene->
							setFlySimulationParametersAccordingToMouseMotion(motionX, motionY);
					}
			else
					{	char key;
						key = mapMouseMotionToAction(motionX, motionY);
						triggerActionAccordingToInterfaceMode(key);

						cout<< "mouse position, x=" << x <<" y=" << y << endl
							<< "mouse motion, motionX=" << motionX 
							<<" motionY=" << motionY << endl
							<< " windowID=" << glutGetWindow() << endl
							<< "Action key=" << key << endl << endl;
					}

			glutPostRedisplay();
		}




//***************************************************************************
///<summary>
/// Section 2.
/// In the following, we define the static member callback functions 
////	to handle menu events from each menu
///</summary>
//**************************************************************************

		//Handler for SELECT_PROTOTYPE_LIST_SUBSUBMENU
		void myGlutCallbackFunctions::handle_SELECT_PROTOTYPE_LIST_SUBSUBMENU(int prototypeID)
		{	if (ptrToCurrentScene == NULL)
				return;
			ptrToCurrentScene->setPrototypeOfCurrentObject(prototypeID);
		}


		//Handler for SELECT_OBJECT_LIST_SUBSUBMENU
		void myGlutCallbackFunctions::handle_SELECT_OBJECT_LIST_SUBSUBMENU(int objectID)
		{	if (ptrToCurrentScene == NULL)
				return;
			ptrToCurrentScene->selectObject(objectID);
			ptrToCurrentScene->
				setCurrentInterfaceInputMode( TO_MOVE_THIS_SELECTED_OBJECT );
		}


		//Handler for DELETE_OBJECT_LIST_SUBSUBMENU
		void myGlutCallbackFunctions::handle_DELETE_OBJECT_LIST_SUBSUBMENU(int objectID)
		{	ptrToCurrentScene->deleteObject(objectID);

			glutSetMenu(ptrToCurrentScene->get_id_DELETE_OBJECT_LIST_SUBSUBMENU() );
			glutRemoveMenuItem( 1+ptrToCurrentScene->totalNumOfObjects());

			glutSetMenu(ptrToCurrentScene->get_id_SELECT_OBJECT_LIST_SUBSUBMENU() );
			glutRemoveMenuItem( 1+ptrToCurrentScene->totalNumOfObjects());
		}


		//Handler for SELECT_CAMERA_LIST_SUBSUBMENU
		void myGlutCallbackFunctions::handle_SELECT_CAMERA_LIST_SUBSUBMENU(int cameraID)
		{	ptrToCurrentScene->selectCamera(cameraID);
			ptrToCurrentScene->
				setCurrentInterfaceInputMode(TO_MOVE_THIS_SELECTED_CAMERA);
		}



		//Handler for INFO_SUBMENU
		void myGlutCallbackFunctions::handle_INFO_SUBMENU(int entryID)
		{
			//************************************************************************
			//The following charater array stores the name of the file
			//	to store OpenGL commands that can be compiled to regenerate 
			//	the current snapshot of the current scene
			//************************************************************************
			char FileName_snapshot_CPP[50] ;
			strcpy(FileName_snapshot_CPP, "SNAPSHOT_VIEW");
			nonnegativeNumToCharsInArray( ptrToCurrentScene->getViewCount(), 
											FileName_snapshot_CPP+5, 45);
			strcat(FileName_snapshot_CPP, "_");
													
			strcat(FileName_snapshot_CPP, ptrToCurrentScene->getSceneTitle());
			strcat(FileName_snapshot_CPP, ".cpp");

			//************************************************************************
			//The following charater array stores the name of the file
			//	to store the information of
			//	cameras and objects of the curent scene
			//************************************************************************
			char  FileName_scene_Objects_Cameras_Info[50]; 
			strcpy(FileName_scene_Objects_Cameras_Info,  "SCENE_INFO_");
			strcat(FileName_scene_Objects_Cameras_Info, ptrToCurrentScene->getSceneTitle());
			strcat(FileName_scene_Objects_Cameras_Info, ".txt");
				
			switch ( entryID )
			{
				case SAVE_CURRENT_SNAPSHOT_AS_OPENGL_COMMANDS:
					
					ptrToCurrentScene->
						saveCurrentSnapshotOfSceneIntoFileAsOpenGLCommands(FileName_snapshot_CPP);
					break;

				case SAVE_SCENE_CAMERAS_OBJECTS_INFO:
					ptrToCurrentScene->
						save_Cameras_Objects_Info_Of_Scene(FileName_scene_Objects_Cameras_Info);
					break;

				case LOAD_SCENE_CAMERAS_OBJECTS_INFO:
					ptrToCurrentScene->
						load_Cameras_Objects_Info_Of_Scene(FileName_scene_Objects_Cameras_Info);

					ptrToCurrentScene->rebuild_ENTIRE_MENU_HIERARCHY();
					break;

				case CREATE_NEW_VIEW_OF_SCENE:
					ptrToCurrentScene->createWindowAttachScene();
					break;

				case DELETE_EXISTING_VIEW_OF_SCENE:
					ptrToCurrentScene->detachFromWindow( glutGetWindow() );
					glutDestroyWindow( glutGetWindow() );
					break;

				default:
					break;
			}
		}


		//********************************************************
		// callback function to handle the object submenu events 
		//***********************************************************/
		void myGlutCallbackFunctions::handle_OBJECTS_SUBMENU(int  entryID)
		{
			switch ( entryID )
			{
				case CURRENT_SELECTED_OBJECT:
					break;

				case CREATE_AND_SELECT_A_NEW_OBJECT:
					ptrToCurrentScene->createAndSelectOneNewObject();
					ptrToCurrentScene->
						setCurrentInterfaceInputMode(TO_SCALE_THIS_SELECTED_OBJECT);

					char tipsOnNewObject[50];
					ptrToCurrentScene->setupTipsOnNewObject(tipsOnNewObject, 50);

					//Add a new entry to the object list subsub menu
					glutSetMenu(ptrToCurrentScene->get_id_SELECT_OBJECT_LIST_SUBSUBMENU());
					glutAddMenuEntry(tipsOnNewObject, 
						ptrToCurrentScene->totalNumOfObjects()-1);

					//Add a new entry to the object list subsub menu
					glutSetMenu(ptrToCurrentScene->get_id_DELETE_OBJECT_LIST_SUBSUBMENU());
					glutAddMenuEntry(tipsOnNewObject, 
						ptrToCurrentScene->totalNumOfObjects()-1);

					break;

				/*  //Repalced by the select camera subsubmenu
					//case SELECT_NEXT_CAMERA: 
				*/
				case DUPLICATE_THIS_SELECTED_OBJECT:
					ptrToCurrentScene->duplicateSelectedObject();
					break;

				case SCALE_THIS_SELECTED_OBJECT:
					ptrToCurrentScene->
						setCurrentInterfaceInputMode(TO_SCALE_THIS_SELECTED_OBJECT);
					break;

				case ROTATE_THIS_SELECTED_OBJECT:
					ptrToCurrentScene->
						setCurrentInterfaceInputMode(TO_ROTATE_THIS_SELECTED_OBJECT);
					break;

				case MOVE_THIS_SELECTED_OBJECT:
					ptrToCurrentScene->
						setCurrentInterfaceInputMode(TO_MOVE_THIS_SELECTED_OBJECT);
					break;

				case CHANGE_THE_COLOR_OF_THIS_SELECTED_OBJECT:
					ptrToCurrentScene->
						setCurrentInterfaceInputMode(TO_CHANGE_THE_COLOR_OF_THIS_SELECTED_OBJECT);
					break;

				default:
					break;
			}
		}


		//********************************************************
		// callback function to handle the camera submenu events 
		//***********************************************************/
		void myGlutCallbackFunctions::handle_CAMERAS_SUBMENU(int entryID)
		{
			switch ( entryID )
			{
				case CURRENT_SELECTED_CAMERA:
					break;
				/*  //Repalced by the select camera subsubmenu
				case SELECT_NEXT_CAMERA: 
					ptrToCurrentScene->selectNextCamera();
					ptrToCurrentScene->setCurrentInterfaceInputMode(TO_MOVE_THIS_SELECTED_CAMERA);
					break;
				*/
				case MOVE_THIS_SELECTED_CAMERA:
					ptrToCurrentScene->setCurrentInterfaceInputMode(TO_MOVE_THIS_SELECTED_CAMERA);
					break;

				case FLIP_CAMERA_TO_OTHER_SIDE_OF_AXIS:
					ptrToCurrentScene->setCurrentInterfaceInputMode(TO_FLIP_CAMERA_TO_OTHER_SIDE_OF_AXIS);
					break;

				case MOVE_LOOKED_AT_POINT_OF_THIS_SELECTED_CAMERA:
					ptrToCurrentScene->
					setCurrentInterfaceInputMode(TO_MOVE_LOOKED_AT_POINT_OF_THIS_SELECTED_CAMERA);
					break;

				case KEEP_LOOKING_AT_THIS_LOOKED_AT_POINT:
					ptrToCurrentScene->keepThisCameraLookingAtThisObject();
					ptrToCurrentScene->setCurrentInterfaceInputMode(TO_MOVE_THIS_SELECTED_CAMERA);
					break;

				case KEEP_LOOKING_ALONG_THIS_DIRECTION:
					ptrToCurrentScene->keepThisCameraLookingAlongThisDirection();
					ptrToCurrentScene->setCurrentInterfaceInputMode(TO_MOVE_THIS_SELECTED_CAMERA);
					break;

				case START_AUTOMATIC_REVOLVING_AROUND_LOOKED_AT_POINT:
					ptrToCurrentScene->startThisCameraAutomaticRevolving();
					ptrToCurrentScene->setCurrentInterfaceInputMode(TO_MOVE_THIS_SELECTED_CAMERA);
					break;

				case STOP_AUTOMATIC_REVOLVING_AROUND_LOOKED_AT_POINT_AND_RETURN_TO_PREVIOUS_LOCATION:
					ptrToCurrentScene->
						return_After_StopThisCameraAutomaticRevolvingAroundTheLookedAtPoint();
					ptrToCurrentScene->setCurrentInterfaceInputMode(TO_MOVE_THIS_SELECTED_CAMERA);
					break;

				case STOP_AUTOMATIC_REVOLVING_AROUND_LOOKED_AT_POINT_AND_STAY_HERE:
					ptrToCurrentScene->
						stay_And_StopThisCameraAutomaticRevolvingAroundTheLookedAtPoint();
					ptrToCurrentScene->setCurrentInterfaceInputMode(TO_MOVE_THIS_SELECTED_CAMERA);
					break;

				case ZOOM_IN_ZOOM_OUT:
					ptrToCurrentScene->setCurrentInterfaceInputMode(TO_ZOOM_IN_ZOOM_OUT);
					break;

				default:
					break;
			}
		}


		//********************************************************
		// callback function to handle the main menu events 
		//***********************************************************/
		void myGlutCallbackFunctions::handle_MAIN_MENU(int entryID)
		{
			switch ( entryID )
			{
				case CURRENT_INTERFACE_INPUT_MODE:
					break;				
				case WORKING_ON_OBJECTS:
					break;
				case WORKING_ON_CAMERAS:
					break;

				case START_FLIGHT_SIMULATION_USING_THIS_CAMERA:
					ptrToCurrentScene->
						setCurrentInterfaceInputMode
						(TO_DO_FLIGHT_SIMULATION_USING_THIS_SELECTED_CAMERA);

					ptrToCurrentScene->startFlightSimulation();
					break;

				case STOP_FLIGHT_SIMULATION_AND_RETURN_TO_PREVIOUS_LOCATION:
					ptrToCurrentScene->stopFlightSimulation_ReturnToPreviousLocation();
					ptrToCurrentScene->
						setCurrentInterfaceInputMode
						(TO_MOVE_THIS_SELECTED_CAMERA);

					break;

				case STOP_FLIGHT_SIMULATION_AND_STAY_HERE:
					ptrToCurrentScene->stopFlightSimulation_StayHere();
					ptrToCurrentScene->
						setCurrentInterfaceInputMode
						(TO_MOVE_THIS_SELECTED_CAMERA);

					break;

				case SHOW_XYZ_AXES:
					ptrToCurrentScene->setToShowAxesXYZ();
					break;
				case HIDE_XYZ_AXES:
					ptrToCurrentScene->setNotToShowAxesXYZ();
					break;			

				case SCENE_INFORMATON:
					break;
				default:
					break;
			}
		}
		//***********************************************************/
		//*** End of the static member callback functions for menu events  *
		//***********************************************************/

