#include "interfacedScene.h"
#include <map>

class myGlutCallbackFunctions
{

	private:
		//*****************************************************************
		// A static class member used to point to the actual scene currently 
		//		attached to the window generating the event.
		// It is used by all the static member callback functions defined here
		//	 to modify the contents of the current scene
		//*****************************************************************
		static interfacedScene * ptrToCurrentScene;

		//*****************************************************************
		// A static member keeping the information of scenes attached to
		//	differnt windows. 
		// For exmaple, map_WindowID_To_ScenePointer[id] will give you
		//	the pointer pointing to the scene attached to the window 
		//	with id as its window ID
		//*****************************************************************
		typedef int WINDOW_ID;
		static map<WINDOW_ID, interfacedScene *> map_WindowID_To_ScenePointer;

		//*****************************************************************
		// A static member keeping the information of the next window
		//	that we should refresh in the next idle event. 
		//*****************************************************************
		static map<WINDOW_ID, interfacedScene *>::iterator 
				itrNextWindowToSceneEntry;

	public:
		//Initialze GLut settings 
		static void initializeGlut(int & argc, char **argv);

		//*****************************************************************
		// A static member function to attach a scene to a window 
		// This scene will then interact through this window
		//	 with all the static member GLUT callback functions  
		//	 defined in myGlutCallbackFunctions.cpp 
		//*****************************************************************
		static void attachSceneToWindow(interfacedScene & myScene, int windowID);


		//*****************************************************************
		// A static member function to detach from a window its associated scene. 
		//*****************************************************************
		static void detachSceneFromWindow(interfacedScene & myScene, int windowID);

		//Get the pointer to the scene attacked to the current window
		static interfacedScene * getPtrToSceneAttachedToCurrentWindow();


		//*************************************************************************
		//	Static member functions implemeted to serve as event handlers (callback functions)
		//		for glut MENU events from various menus of the interface
		//  These functions then interact with the interfaceScene object 
		//		attached to the current window.
		//*************************************************************************
		static void handle_SELECT_PROTOTYPE_LIST_SUBSUBMENU(int prototypeID);

		static void handle_SELECT_OBJECT_LIST_SUBSUBMENU(int objectID);

		static void handle_DELETE_OBJECT_LIST_SUBSUBMENU(int objectID);

		static void handle_SELECT_CAMERA_LIST_SUBSUBMENU(int cameraID);

		static void handle_INFO_SUBMENU(int entryID);

		static void handle_OBJECTS_SUBMENU(int  entryID);

		static void handle_CAMERAS_SUBMENU(int entryID);

		static void handle_MAIN_MENU(int entryID);


		//*****************************************************************************
		//	Gobal functions implemented to serve as event handlers (callback functions)
		//		to handle STANDARD GLUT EVENTS like 
		//		display, reshape, keyboard, idle, mouse ...
		//  These functions then interact with the interfaceScene object 
		//		attached to the current window.
		//*****************************************************************************
		static void display();

		static void keyboard(unsigned char key, int x, int y);

		static void reshape(int w, int h);

		static void idle();

		static void mouse(int button, int state, int x, int y);

		static void mouseMotion(int x, int y);


		//*************************************************************
		//Constants Mapping keys to actions of increment or decrement
		//along the X,Y Z coordinate axes,  RGB color axes, and 
		//gluPerspective fov angle
		//*************************************************************
		static const char KEY_TO_NO_ACTION;
		static const char KEY_TO_UNKNOWN_ACTION;

		static const char KEY_TO_SELECT_NEXT_OBJECT;
		static const char KEY_TO_SELECT_NEXT_CAMERA;
		static const char KEY_TO_SELECT_NEXT_INTERFACE_INPUT_MODE;

		static const char X_up;
		static const char X_down;
		static const char Y_up;
		static const char Y_down;
		static const char Z_up;
		static const char Z_down;

		static const char R_up;
		static const char R_down;
		static const char G_up;
		static const char G_down;
		static const char B_up;
		static const char B_down;

		static const char FOV_up1;
		static const char FOV_down1;
		static const char FOV_up2;
		static const char FOV_down2;
		static const char FOV_up3;
		static const char FOV_down3;

		//************************************************************************
		//Definging basic unit of change 
		//along the X,Y Z coordinate axes and RGB color axes
		//*************************************************************
		static const float UNIT_SCALE_IN_XYZ;
		static const float UNIT_ROTATE_IN_XYZ;
		static const float UNIT_MOVE_IN_XYZ;
		static const float UNIT_COLOR_CHANGE_IN_RGB;

	private:

		//Map the mouse motion into a key representing an action
		static char mapMouseMotionToAction(double mouseMotionX, double mouseMotionY);

		//Trigger an action according to what is the key and 
		//	what is the current interface mode
		static void triggerActionAccordingToInterfaceMode(unsigned char key);


};