
#include <vector>
#include <string>
#include <iostream>
#include <fstream>
#include <cmath>
#include <cstring>
using namespace std;

#include <GL/glut.h>
#include "scene.h"

scene::scene()
{
	objectsVector.push_back( geometricObject() );

	myCameras[0].setCameraNameAndProjectionStyle("Camera A: glOtho", camera::OTHO);
	myCameras[1].setCameraNameAndProjectionStyle("Camera B: glOtho", camera::OTHO);
	myCameras[2].setCameraNameAndProjectionStyle("Camera C: glPerspective", camera::PERSPECTIVE);
	myCameras[3].setCameraNameAndProjectionStyle("Camera D: glPerspective", camera::PERSPECTIVE);

	renderCount = 0;
	needToShowAxesXYZ = true;

	//setUpDefaultSceneLighting();

}



void scene::drawAxesXYZ()
{	//Draw XYZ axes
	glBegin(GL_LINES);
	
		glColor3f(1,0,0);
		glVertex3f(0,0,0);
		glVertex3f(4,0,0);

		glColor3f(0,1,0);
		glVertex3f(0,0,0);
		glVertex3f(0,4,0);

		glColor3f(0,0,1);
		glVertex3f(0,0,0);
		glVertex3f(0,0,4);

	glEnd();

	//Show the X, Y, Z letters
	glColor3f(1,0,0);
	glRasterPos3f(5, 0, 0);
	glutBitmapCharacter(GLUT_BITMAP_9_BY_15, 'X');

	glColor3f(0, 1, 0);
	glRasterPos3f(0, 5, 0);
	glutBitmapCharacter(GLUT_BITMAP_9_BY_15, 'Y');

	glColor3f(0, 0, 1);
	glRasterPos3f(0, 0, 5);
	glutBitmapCharacter(GLUT_BITMAP_9_BY_15, 'Z');

}


void scene::displayInterfaceModeMessage()
{
	glDisable(GL_LIGHTING);

	if (textMessage == NULL)
		textMessage = "NO SCENE MESSAGE";

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();

	int i;
	float xPos, yPos, zPos;
	myCameras[ID_SelectedCamera[glutGetWindow()] ].getRasterPositionLeftTopCorner(xPos, yPos, zPos);
	glColor3f(1, 0, 1);
	glRasterPos3f(xPos, yPos, zPos);

	for (i=0; i<strlen(textMessage); ++i)
		glutBitmapCharacter(BITMAP_FONT_MEDIUM, textMessage[i]);
	

	glPopMatrix();
}

//*********************  A useful support function ****************************
//Try to convert a non-negative interger number into a C_style character string
//	in myCahtArray that can hold arraySize characters.
//*********** Used in the two member functions following it *****************
bool scene::nonnegativeNumToCharsInArray(int number, char * myCharArray, int arraySize)
{
	if (number < 0)
	{	myCharArray[0] = '\0';
		return false;
	}

	int temp = number;
	int length;		//Calculate the number of digits required
	for (length = 1; temp > 9; temp =temp/10)
		++length;

	if (length+ 1 > arraySize )
	{	myCharArray[0] = '\0';
		return false;
	}


	myCharArray[length]='\0';


	int index = length-1;
	if (number == 0)
		myCharArray[index]='0';
	else
		for (temp = number; temp > 0; temp = temp / 10)
		{	myCharArray[index] = '0' + (temp % 10);
			--index;
		}

	return true;
}



void scene::displayObjectCameraMessage()
{
	glDisable(GL_LIGHTING);

	glPushMatrix();
	glLoadIdentity();

	char selectedObjectCameraMessage[80] = "OBJECT ";
	char numberArray[6];

	nonnegativeNumToCharsInArray(ID_SelectedObject[ glutGetWindow() ] , numberArray, 6);
	strcat(selectedObjectCameraMessage,numberArray);

	strcat(selectedObjectCameraMessage," selected, CAMERA ");

	int i = strlen(selectedObjectCameraMessage);
	selectedObjectCameraMessage[ i ]
		= 'A' + ID_SelectedCamera[ glutGetWindow() ];
	selectedObjectCameraMessage[ i + 1 ] = '\0';

	strcat(selectedObjectCameraMessage," selected");


	float xPos, yPos, zPos;
	myCameras[ID_SelectedCamera[glutGetWindow()] ].getRasterPositionLeftBottomCorner(xPos, yPos, zPos);
	glColor3f(1, 0, 1);
	glRasterPos3f(xPos, yPos, zPos);

	for (i=0; i<strlen(textMessage); ++i)
		glutBitmapCharacter(BITMAP_FONT_MEDIUM, selectedObjectCameraMessage[i]);

	glPopMatrix();
}



void scene::initializeSceneLightingAndMaterialProperties()
{
	//************************************************************************
	//****  Set uo the position of the light and the direction of light  *****
	//************************************************************************
	static const float LIGHT_POSITION[] = {10, 10, 50, 1.0};
	static const float LIGHT_SPOT_DIRECTION[] = {-1, -1, -1};
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		glLightfv(GL_LIGHT0, GL_POSITION, LIGHT_POSITION);
		glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, LIGHT_SPOT_DIRECTION);

	//************************************************************
	//***************  Set up light models ***********************
	//************************************************************
	GLfloat lmodel_ambient[] = { 0.4, 0.4, 0.4, 1.0 };
	GLfloat no_local_view[] = { 0.0 };
		glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
		glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, no_local_view);
		glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);


	//************************************************************
	//**********  Set up properties of material        ***********
	//************************************************************
	static GLfloat no_mat[] = { 0.0, 0.0, 0.0, 1.0 };
	static GLfloat mat_ambient[]	= { 1.0, 1.0, 1.0, 1.0};
	static GLfloat mat_diffuse[]	= { 1.0, 1.0, 1.0, 1.0};
	//static GLfloat mat_specular[]	= { 1.0, 1.0, 1.0, 1.0};
	static GLfloat mat_specular[]	= { 0.4, 0.4, 0.4, 1.0};
	static GLfloat mat_emission[]	= { 0.1, 0.1, 0.1, 1.0};

	static GLfloat no_shininess[] = { 0.0 };
	static GLfloat low_shininess[] = { 15.0 };
	static GLfloat high_shininess[] = { 50.0 };

		glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
		glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
		glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
		glMaterialfv(GL_FRONT, GL_EMISSION, no_mat);
		glMaterialfv(GL_FRONT, GL_SHININESS, high_shininess);


	//************************************************************
	//********  Set up smooth shade model & depth test   *********
	//************************************************************
	glShadeModel(GL_SMOOTH);
	glEnable(GL_DEPTH_TEST);


	//************************************************
	//********  Enable light and GL_LIGHT0   *********
	//************************************************
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
}


void scene::setUpNextCursorStatus()
{
	++renderCount;
	if ( renderCount > 2 * (500/CURSOR_BLINKING_RATE) )
	{	renderCount = 0;
		//cout << "Object selected: Object ID = " << ID_SelectedObject[glutGetWindow()] << endl;
		//cout << "Camera selected: Camera ID = " << ID_SelectedCamera[glutGetWindow()] << endl;
	}
}


bool scene::isCursorOn()
{	return	( renderCount > (500/CURSOR_BLINKING_RATE) );
}


void scene::displayCursorAndReferenceFrame()
{
	glDisable(GL_LIGHTING);

	setUpNextCursorStatus();
	if (needToShowAxesXYZ)
	{
		if ( isCursorOn() )
		{	//Show cursor
			glPushMatrix();
			glColor4f(1.0, 1.0, 1.0, 0.1);
			objectsVector[ID_SelectedObject[glutGetWindow()]].doOpenGLModelTransformations();
			objectsVector[ID_SelectedObject[glutGetWindow()]].renderCursorInOpenGL();
			glPopMatrix();
		}
		else
		{	//Show the XYZ axes of the world
			//and also the original referece frame of the prototype of the selected object
			//if needToShowOriginalRefereceFrameOfSelectedObject is true;
			drawAxesXYZ();

			if (needToShowOriginalRefereceFrameOfSelectedObject == true)
			{	glPushMatrix();
				objectsVector[ID_SelectedObject[glutGetWindow()]].doOpenGLModelTransformations();
				objectsVector[ID_SelectedObject[glutGetWindow()]].rendeOriginalRefereceFrameOfPrototype();
				glPopMatrix();
			}
		}
	}

}

void scene::renderSceneSeenFromCurrentCamera()
{
	//clear frame buffer and z buffer,
	glClearColor(0.0, 0.0, 0.0, 1.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	//Set up the camera location and orientation
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	myCameras[ID_SelectedCamera[glutGetWindow()]].do_OpenGL_LookAt_BasedOnThisCamera();

	//Render each object
	for (unsigned int i=0; i<objectsVector.size(); ++i)
	{	
		glPushMatrix();

		objectsVector[i].setOpenGLColorAndMaterialProperties();
		
		objectsVector[i].doOpenGLModelTransformations();

		objectsVector[i].renderPrototypeInOpenGL();

		glPopMatrix();
	}

	displayObjectCameraMessage();
	displayInterfaceModeMessage();
	displayCursorAndReferenceFrame();

	glutSwapBuffers();

	glFlush();

}


bool scene::saveCurrentSnapshotOfSceneIntoFileAsOpenGLCommands(const char * fileName)
{	
	cout << endl << endl 
		 << "****************************************" << endl
		 << "Preparing to save OpenGL commands to regenerate the scene " << endl
		 << "to a file named as: " << fileName << "  ..." << endl;

	ofstream outfile;
	outfile.open(fileName);
	if (!outfile.is_open())
		return false;

	outfile << "#include <GL/glut.h>" << endl
			<< endl 
			<< endl
			<< "//*****************************************************" << endl
			<< "//A gloabal GLUquadricObj object dynamically allocated " << endl
			<< "//	for drawing GLU guadric objects in display() "		 << endl
			<< "//*****************************************************" << endl
			<< endl
			<< "GLUquadricObj * ptrToQuadricInfo;"		<< endl
			<< "#define NUM_SLICES 40"		<< endl
			<< "#define NUM_STACKS 40"		<< endl
			<< endl << endl
			<< "//***************************************************************" << endl
			<< "//OpenGL commands to regenerate the scene saved in two functions:" << endl
			<< "//		display() and reshape( int w, int h)" << endl
			<< "//***************************************************************" << endl
			<< endl 
			<< endl;

	myCameras[ID_SelectedCamera[glutGetWindow()]].
		saveToFile_OpenGL_Projection_BasedOnThisCamera(outfile);

	outfile << endl << endl
			<< "void display()" << endl
			<< "{" << endl
			<< " glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);" << endl
			<< endl
			<< " glMatrixMode(GL_MODELVIEW);" << endl
			<< " glLoadIdentity();" << endl;

	myCameras[ID_SelectedCamera[glutGetWindow()]].
		saveToFile_OpenGL_LookAt_BasedOnThisCamera(outfile);

	for (unsigned int i=0; i<objectsVector.size(); ++i)
	{	
		outfile << endl
				<< " //***************************************************" << endl
				<< " // to generate geometric object " << i  << endl
				<< " //***************************************************" << endl
				<< " glPushMatrix();" << endl 
				<< endl;

		objectsVector[i].saveToFile_SetOpenGLColor(outfile);
		objectsVector[i].saveToFile_OpenGLModelTransformations(outfile);
		objectsVector[i].saveToFile_PrototypeInOpenGL(outfile);

		outfile << endl 
				<< " glPopMatrix();" 
				<< endl;
	}

	outfile 
		<< " glFlush();" << endl
		<< "}" << endl
		<< "//************End of display function *****************" << endl;

	//Save the main function 
	outfile << endl << endl
			<< "void main(int argc, char **argv)" << endl
			<< "{"	<< endl
			<< " glutInit(&argc, argv);" << endl
			<< " glutInitWindowSize(500, 500);" << endl
			<< " glutCreateWindow(\"Snapshot of my Scene\");" << endl
			<< " glutReshapeFunc(reshape);" << endl
			<< " glutDisplayFunc(display);" << endl
			<< " glClearColor(0.0, 0.0, 0.0, 1.0);" << endl
			<< endl
			<< " ptrToQuadricInfo = gluNewQuadric();" << endl
			<< " glutMainLoop();" << endl
			<< "}" << endl
			<< endl;

	outfile.close();

	cout << endl << endl 
		<< "****************************************" << endl
		<< "OpenGL commands to regenerate the current view of the scene  " << endl
		<< "All saved in : " << fileName << endl;

	return true;
}

bool scene::save_Cameras_Objects_Info_Of_Scene(const char * fileName)
{	cout << endl << endl 
		 << "****************************************" << endl
		 << "Preparing to save info of cameras and objects of the scene " << endl
		 << "to a file named as: " << fileName << "  ..." << endl;

	ofstream outfile;
	outfile.open(fileName);
	if (!outfile.is_open())
		return false;

	// store the number of cameras
	outfile << NumOfCameras << endl; 

	// store info of individual cameras
	for (unsigned int i=0; i<NumOfCameras; ++i)
		myCameras[i].saveToFile_CameraInfo(outfile);

	// store the number of objects
	outfile << objectsVector.size() << endl; 

	// store info of individual objects
	for (unsigned int i=0; i<objectsVector.size(); ++i)
		objectsVector[i].saveToFile_ObjectInfo(outfile);

	outfile.close();

	cout << endl << endl 
		<< "****************************************" << endl
		<< "Info of cameras and objects of the scene " << endl
		<< "all saved into to the file: " << fileName << "  ..." << endl;

	return true;
}


bool scene::load_Cameras_Objects_Info_Of_Scene(const char * fileName)
{	cout << endl << endl 
		 << "****************************************" << endl
		 << "Preparing to load info of cameras and objects of the scene " << endl
		 << "from the file: " << fileName << "  ..." << endl;

	ifstream infile;
	infile.open(fileName);
	if (!infile.is_open())
		return false;
	
	// read the number of cameras
	int infileNumOfCameras;  
	infile >> infileNumOfCameras; 

	for (unsigned int i=0; i<NumOfCameras && i < infileNumOfCameras; ++i)
		myCameras[i].loadFromFile_CameraInfo(infile);


	int numOfObjects;
	infile >> numOfObjects;

	cout << endl << endl 
		 << "****************************************" << endl
		 << "Cameras loading is complete. " 
		 << "Start loading " << numOfObjects << " objects" <<endl<< endl;



	objectsVector.clear();
	for (unsigned int i=0; !infile.eof() && i<numOfObjects; ++i)
	{	geometricObject tempObj;
		tempObj.loadFromFile_ObjectInfo(infile);
		objectsVector.push_back(tempObj);
	}

	cout << endl << endl 
		 << "****************************************" << endl
		 << "objects loading complete " << endl<< endl;

	infile.close();

	cout << endl << endl 
	<< "****************************************" << endl
	<< "Info of cameras and objects of the scene " << endl
	<< "all loaded from the file: " << fileName << "  ..." << endl;

	return true;
}


void scene::updateScene()
{	if ( myCameras[ID_SelectedCamera[glutGetWindow()]].
		   isCameraInAutomaticRevolvingMode() == true 
	   )
			{	myCameras[ID_SelectedCamera[glutGetWindow()]].
					revolveOneStepAroundTheLookedAtPoint();
			}
	else
	if ( myCameras[ID_SelectedCamera[glutGetWindow()]].
		   isCameraInFlightSimulationMode() == true 
	   )
			{	myCameras[ID_SelectedCamera[glutGetWindow()]].
					flyOneStepInFlightSimulation();
			}

	glutPostRedisplay();
}

void scene::doOpenGLReshape(int w, int h)
{
    glViewport(0, 0, w, h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

	myCameras[ID_SelectedCamera[glutGetWindow()]].
		do_OpenGL_Projection_BasedOnThisCamera(w, h);

    glMatrixMode(GL_MODELVIEW);
}

void scene::reshapeCameraView()
{
	doOpenGLReshape(glutGet(GLUT_WINDOW_WIDTH),glutGet(GLUT_WINDOW_HEIGHT));
	glutPostRedisplay();

}

void scene::createAndSelectOneNewObject()
{
	objectsVector.push_back(geometricObject());
	ID_SelectedObject[glutGetWindow()] = objectsVector.size()-1;
}


void scene::duplicateSelectedObject()
{	static float smallIncrement = 0.01;

	int id_thisObject = ID_SelectedObject[ glutGetWindow() ];
	geometricObject tempObject = objectsVector[ id_thisObject ];
	tempObject.moveAlongAxis(	(XYZ_AXIS) (rand() % 3 ), 
								0.1*(-25 + rand()%51)+ smallIncrement
							);
	smallIncrement += 0.03;

	objectsVector.
		push_back( tempObject );
}


bool scene::deleteObject(int index)
{ if ( index<0 || index >= objectsVector.size())
	return false;		//out of range

  objectsVector.erase(objectsVector.begin()+index);

  if (index < ID_SelectedObject[glutGetWindow()])
	ID_SelectedObject[glutGetWindow()] = ID_SelectedObject[glutGetWindow()] -1;

  if (index == ID_SelectedObject[glutGetWindow()] && 
	  ID_SelectedObject[glutGetWindow()] == objectsVector.size()
	 )
	ID_SelectedObject[glutGetWindow()] = ID_SelectedObject[glutGetWindow()] -1;

  return true;
}

void scene::selectNextObject()
{	if ( objectsVector.size() > 1)
		ID_SelectedObject[glutGetWindow()] = (ID_SelectedObject[glutGetWindow()] + 1) % ( objectsVector.size() );
	glutPostRedisplay();
}

bool scene::selectObject(int i)
{	if (  (i<0) || (i>= objectsVector.size()) ) 
		return false;
	ID_SelectedObject[glutGetWindow()] = i;
	return true;
}


void scene::setPrototypeOfCurrentObject(int prototypeID)
{
	(objectsVector[ ID_SelectedObject[glutGetWindow()] ]).setPrototype(prototypeID);
}

void scene::selectNextCamera()
{	ID_SelectedCamera[glutGetWindow()] = (ID_SelectedCamera[glutGetWindow()] + 1) % NumOfCameras;
	reshapeCameraView();
}


bool scene::selectCamera(int i)
{	if (i<0 || i>=NumOfCameras) 
		return false;
	ID_SelectedCamera[glutGetWindow()] = i;
	reshapeCameraView();
	return true;
}

void scene::keepThisCameraLookingAlongThisDirection()
{	myCameras[ID_SelectedCamera[glutGetWindow()]].
		keepCameraLookingAtTheSameDirection();
}

void scene::keepThisCameraLookingAtThisObject()
{	float x,y,z;
	objectsVector[ID_SelectedObject[glutGetWindow()]].
		getObjectCoordinates(x, y, z);
	myCameras[ID_SelectedCamera[glutGetWindow()]].
		turnCameraToLookAtThisPoint(x, y, z);
	myCameras[ID_SelectedCamera[glutGetWindow()]].
		keepCameraLookingAtTheSamePoint();
}

void scene::startThisCameraAutomaticRevolving()
{	myCameras[ID_SelectedCamera[glutGetWindow()]].
		startCameraAutomaticRevolvingAroundTheLookedAtPoint();
}


void scene::return_After_StopThisCameraAutomaticRevolvingAroundTheLookedAtPoint()
{
	myCameras[ID_SelectedCamera[glutGetWindow()]].
		stopCameraAutomaticRevolvingAroundTheLookedAtPoint_ReturnToPreviousLocation();
}


void scene::stay_And_StopThisCameraAutomaticRevolvingAroundTheLookedAtPoint()
{
	myCameras[ID_SelectedCamera[glutGetWindow()]].
		stopCameraAutomaticRevolvingAroundTheLookedAtPoint_StayHere();
}


void scene::moveThisObjectAlongAxis(XYZ_AXIS axis, float units)
	{objectsVector[ID_SelectedObject[glutGetWindow()]].moveAlongAxis(axis, units);
	}

void scene::rotateThisObjectAlongAxis(XYZ_AXIS axis, float degrees)
	{objectsVector[ID_SelectedObject[glutGetWindow()]].rotateAlongAxis(axis, degrees);
	}

void scene::scaleThisObjectAlongAxis(XYZ_AXIS axis, float fraction)
	{objectsVector[ID_SelectedObject[glutGetWindow()]].scaleAlongAxis(axis, fraction);
	}

void scene::scaleThisObjectColorAlongAxis(COLOR_AXIS axis, float fraction)
	{objectsVector[ID_SelectedObject[glutGetWindow()]].scaleColorAlongAxis(axis, fraction);			
	}

void scene::moveThisCameraAlongAxis(XYZ_AXIS axis, float units)
	{myCameras[ID_SelectedCamera[ glutGetWindow() ]].moveCameraAlongAxis(axis, units);
	}

void scene::flipThisCameraToOtherSideOfAxis(XYZ_AXIS axis)
	{myCameras[ID_SelectedCamera[ glutGetWindow() ]].
			flipCameraToOtherSideOfAxis(axis);
	}

void scene::moveLookedAtPointOfThisCameraAlongAxis(XYZ_AXIS axis, float units)
	{myCameras[ID_SelectedCamera[ glutGetWindow() ]].moveLookedAtPointAlongAxis(axis, units);
	}

void scene::resetThisCameraPosition()
	{myCameras[ID_SelectedCamera[ glutGetWindow() ]].resetCameraPosition();
		reshapeCameraView();
	}

void scene::zoomInThisCameraView()
	{myCameras[ID_SelectedCamera[ glutGetWindow() ]].zoomInCameraView();
		reshapeCameraView();
	}

void scene::zoomOutThisCameraView()
	{myCameras[ID_SelectedCamera[ glutGetWindow() ]].zoomOutCameraView();
		reshapeCameraView();
	}

int scene::currentObject()
	{return ID_SelectedObject[ glutGetWindow() ];
	}

int scene::currentCamera() 
	{return ID_SelectedCamera[ glutGetWindow() ];
	}

void scene::detachSelectedObjectAndCameraFromWindow(int windowID)
{
	ID_SelectedObject.erase(windowID);
	ID_SelectedCamera.erase(windowID);
}

//Given the world coordinates x,y,z of a point
//Determine the window coordinates winX, winY, winZ of the point
//	based on the seeting of the camera selected by current window
//	and the window itself
bool scene::projectToScreenCoordinates
	(	double x,			double y,		double z,
		double& winX,		double& winY,	double& winZ
	)
{
	if (glutGetWindow() <= 0)
		return false;

	if	(ID_SelectedCamera.find( glutGetWindow() )
			== ID_SelectedCamera.end()
		)
		return false;

	if	(	(ID_SelectedCamera[ glutGetWindow() ] < 0 )	||
			(ID_SelectedCamera[ glutGetWindow() ] 
				>= scene::NumOfCameras
			)
		)
		return false;


	return myCameras[ ID_SelectedCamera[ glutGetWindow() ] ].projectToScreenCoordinates
			(	glutGet(GLUT_WINDOW_WIDTH), glutGet(GLUT_WINDOW_HEIGHT), 
				x,		y,		z,
				winX,	winY,	winZ
			);
}

