#ifndef SCENE_H
#define SCENE_H

#include <GL/glut.h>
#include <cmath>
#include <vector>
#include <map>
#include <set>
#include <iostream>
using namespace std;

#include "geometricObject.h"
#include "camera.h"

// enum XYZ_AXIS {X=0, Y=1, Z=2};	//Defined in "geometricObject.h"
// enum COLOR_AXIS {R=0, G=1, B=2};	//Defined in "geometricObject.h"

#define	BITMAP_FONT_SMALL		GLUT_BITMAP_HELVETICA_10
#define	BITMAP_FONT_MEDIUM		GLUT_BITMAP_HELVETICA_12
#define	BITMAP_FONT_LARGE		GLUT_BITMAP_HELVETICA_18

class scene
{	private:
		vector<geometricObject> objectsVector; 

		typedef int WINDOW_ID;	
		typedef int OBJECT_ID;
		map<WINDOW_ID, OBJECT_ID>  ID_SelectedObject;
		//ID_SelectedObject is map from WINDOW_ID to OBJECT_ID
		//	ID_SelectedObject[windowID] will give you the id of the object selected by 
		//	the window with the ID windowID.
		//In particular, ID_SelectedObject[ glutGetWindow() ] will give you the id of  
		//	the object selected by the current window.


		static const int NumOfCameras = 4;
		static const int CURSOR_BLINKING_RATE = 5;

		camera myCameras[NumOfCameras];

		//typedef int WINDOW_ID;	//defined already
		typedef int CAMERA_ID;
		map<WINDOW_ID, CAMERA_ID> ID_SelectedCamera;
		//ID_SelectedCamera is a map from WINDOW_ID to CAMERA_ID
		//	ID_SelectedCamera[windowID] will give you the id of the camera selected by 
		//	the window with the ID windowID.
		//In particular, ID_SelectedCamera[ glutGetWindow() ] will give you the id of  
		//	the camera selected by the current window.


		// typedef int OBJECT_ID; //defined already
		typedef set<OBJECT_ID> group;
		vector<group> objectGroups;

		typedef int GROUP_ID;
		map<WINDOW_ID, GROUP_ID> ID_SelectedGroup;
		//ID_SelectedGroup is a map from WINDOW_ID to Group_ID
		//	ID_SelectedCamera[windowID] will give you the id of the group selected by 
		//	the window with the ID windowID.
		//In particular, ID_SelectedGroup[ glutGetWindow() ] will give you the id of  
		//	the group selected by the current window.


		int renderCount;
		bool needToShowAxesXYZ;
		bool needToShowOriginalRefereceFrameOfSelectedObject;

		
		//************************************************
		//Text message to be displayed on the window
		//************************************************
		const char * textMessage;

	public: 
		//constructor
		scene();

		void doOpenGLReshape(int w, int h);

		void reshapeCameraView();

		void updateScene();

		void renderSceneSeenFromCurrentCamera();

		void setPrototypeOfCurrentObject(int prototypeID);

		void selectNextCamera();

		bool selectCamera(int i);

		void selectNextObject();

		bool selectObject(int i);

		void createAndSelectOneNewObject();

		void duplicateSelectedObject();

		bool deleteObject(int index);

		void moveThisObjectAlongAxis(XYZ_AXIS axis, float units);
			
		void rotateThisObjectAlongAxis(XYZ_AXIS axis, float degrees);

		void scaleThisObjectAlongAxis(XYZ_AXIS axis, float fraction);

		void scaleThisObjectColorAlongAxis(COLOR_AXIS axis, float fraction);

		void moveThisCameraAlongAxis(XYZ_AXIS axis, float units);

		void flipThisCameraToOtherSideOfAxis(XYZ_AXIS axis);

		void moveLookedAtPointOfThisCameraAlongAxis(XYZ_AXIS axis, float units);

		void resetThisCameraPosition();

		void zoomInThisCameraView();

		void zoomOutThisCameraView();

		void keepThisCameraLookingAtThisObject();

		void keepThisCameraLookingAlongThisDirection();

		void startThisCameraAutomaticRevolving();

		void return_After_StopThisCameraAutomaticRevolvingAroundTheLookedAtPoint();

		void stay_And_StopThisCameraAutomaticRevolvingAroundTheLookedAtPoint();

		void setToShowAxesXYZ() {needToShowAxesXYZ = true; };

		void setNotToShowAxesXYZ() {needToShowAxesXYZ = false; };

		void setToShowOriginalRefereceFrameOfSelectedObject() 
		 {needToShowOriginalRefereceFrameOfSelectedObject = true; };

		void setNotToShowOriginalRefereceFrameOfSelectedObject() 
		 {needToShowOriginalRefereceFrameOfSelectedObject = false; };

		int currentObject();

		int currentCamera();

		int totalNumOfObjects() {return objectsVector.size();};

		int totalNumOfCameras() {return NumOfCameras;};

		bool nonnegativeNumToCharsInArray(int number, char * myCharArray, int arraySize);

		void setInterfaceModeMessage(const char * aMessage)
			{textMessage = aMessage;};

		void displayInterfaceModeMessage();

		void displayObjectCameraMessage();

		void displayCursorAndReferenceFrame();

		bool saveCurrentSnapshotOfSceneIntoFileAsOpenGLCommands(const char * fileName);

		bool save_Cameras_Objects_Info_Of_Scene(const char * fileName);

		bool load_Cameras_Objects_Info_Of_Scene(const char * fileName);

		//Given the point (x,y,z),
		//Calculate its projected point (winX, winY, winZ) on the screen
		//	of the current wondow, based the camera selected by this window
		bool projectToScreenCoordinates
		(	double x,			double y,		double z,
			double& winX,		double& winY,	double& winZ
		);

		//Map the mouse motion into movement of camera location
		void moveThisCameraAccordingToMouseMotion(double mouseMotionX, double  mouseMotionY)
		{	myCameras[ ID_SelectedCamera[ glutGetWindow() ] ].
				moveCameraAccordingToMouseMotion(mouseMotionX, mouseMotionY);
		}


		void initializeSceneLightingAndMaterialProperties();

		void setFlySimulationParametersAccordingToMouseMotion(double mouseMotionX, double  mouseMotionY)
		{	myCameras[ ID_SelectedCamera[ glutGetWindow() ] ].
				setFlySimulationParametersAccordingToMouseMotion(mouseMotionX, mouseMotionY);
		};

		bool isThisCameraInFlightSimulationMode();

		void startFlightSimulation()
		{	myCameras[ ID_SelectedCamera[ glutGetWindow() ] ].
				startFlightSimulation(10, -10, 10, -10, 10, -10);
		};

		void stopFlightSimulation_ReturnToPreviousLocation()
		{	myCameras[ ID_SelectedCamera[ glutGetWindow() ] ].
				stopFlightSimulation_ReturnToPreviousLocation();
		};

		void stopFlightSimulation_StayHere()
		{	myCameras[ ID_SelectedCamera[ glutGetWindow() ] ].
				stopFlightSimulation_StayHere();
		};


	private:
		void setUpNextCursorStatus();

		bool isCursorOn();

		void drawAxesXYZ();

	protected:
		void detachSelectedObjectAndCameraFromWindow(int windowID);

};

#endif
